import copy
from abjad import enums
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.utilities import Infinity
from abjad.utilities import NegativeInfinity


class Timespan(AbjadValueObject):
    """
    Timespan.

    ..  container:: example

        >>> timespan_1 = abjad.Timespan(0, 10)
        >>> timespan_2 = abjad.Timespan(5, 12)
        >>> timespan_3 = abjad.Timespan(-2, 2)
        >>> timespan_4 = abjad.Timespan(10, 20)

    Timespans are closed-open intervals.

    Timespans are immutable.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Timespans'

    __slots__ = (
        '_start_offset',
        '_stop_offset',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(self, start_offset=NegativeInfinity, stop_offset=Infinity):
        if start_offset is None:
            start_offset = NegativeInfinity
        if stop_offset is None:
            stop_offset = Infinity
        start_offset = self._initialize_offset(start_offset)
        stop_offset = self._initialize_offset(stop_offset)
        assert start_offset <= stop_offset, repr((start_offset, stop_offset))
        self._start_offset = start_offset
        self._stop_offset = stop_offset

    ### SPECIAL METHODS ###

    def __and__(self, argument):
        """
        Logical AND of two timespans.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)
            >>> timespan_4 = abjad.Timespan(10, 20)

            >>> timespan_1 & timespan_2
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

            >>> timespan_1 & timespan_3
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))])

            >>> timespan_1 & timespan_4
            TimespanList([])

            >>> timespan_2 & timespan_3
            TimespanList([])

            >>> timespan_2 & timespan_4
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

            >>> timespan_3 & timespan_4
            TimespanList([])

        Returns timespan list.
        """
        import abjad
        argument = self._get_timespan(argument)
        if not self.intersects_timespan(argument):
            return abjad.TimespanList()
        new_start_offset = max(self._start_offset, argument.start_offset)
        new_stop_offset = min(self._stop_offset, argument.stop_offset)
        timespan = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return abjad.TimespanList([timespan])

    def __eq__(self, argument):
        """
        Is true when ``argument`` is a timespan with equal offsets.

        ..  container:: example

            >>> abjad.Timespan(1, 3) == abjad.Timespan(1, 3)
            True

            >>> abjad.Timespan(1, 3) == abjad.Timespan(2, 3)
            False

        Returns true or false.
        """
        return super().__eq__(argument)

    def __format__(self, format_specification=''):
        """
        Formats timespan.

        ..  container:: example

            >>> timespan = abjad.Timespan(0, 10)
            >>> abjad.f(timespan)
            abjad.Timespan(
                start_offset=abjad.Offset(0, 1),
                stop_offset=abjad.Offset(10, 1),
                )

        Returns string.
        """
        import abjad
        if format_specification in ('', 'storage'):
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __ge__(self, argument):
        """
        Is true when ``argument`` start offset is greater or equal
        to timespan start offset.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)

            >>> timespan_2 >= timespan_3
            True

            >>> timespan_1 >= timespan_2
            False

        Returns true or false.
        """
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset >= expr_start_offset:
                return True
            elif (self._start_offset == expr_start_offset and
                self._stop_offset >= expr_stop_offset):
                return True
            return False
        return self._start_offset >= expr_start_offset

    def __gt__(self, argument):
        """
        Is true when ``argument`` start offset is greater than
        timespan start offset.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)

            >>> timespan_2 > timespan_3
            True

            >>> timespan_1 > timespan_2
            False

        Returns true or false.
        """
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset > expr_start_offset:
                return True
            elif (self._start_offset == expr_start_offset and
                self._stop_offset > expr_stop_offset):
                return True
            return False
        return self._start_offset > expr_start_offset

    def __hash__(self):
        """
        Hashes timespan.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    def __illustrate__(self, range_=None, scale=None):
        """
        Illustrates timespan.

        Returns LilyPond file.
        """
        import abjad
        timespans = abjad.TimespanList([self])
        return timespans.__illustrate__(
            range_=range_,
            scale=scale,
            )

    def __le__(self, argument):
        """
        Is true when ``argument`` start offset is less than or equal to
        timespan start offset.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)

            >>> timespan_2 <= timespan_3
            False

            >>> timespan_1 <= timespan_2
            True

        Returns true or false.
        """
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset <= expr_start_offset:
                return True
            elif (self._start_offset == expr_start_offset and
                self._stop_offset <= expr_stop_offset):
                return True
            return False
        return self._start_offset <= expr_start_offset

    def __len__(self):
        """
        Defined equal to ``1`` for all timespans.

        ..  container:: example

            >>> timespan = abjad.Timespan(0, 10)

            >>> len(timespan)
            1

        Returns positive integer.
        """
        return 1

    def __lt__(self, argument):
        """
        Is true when ``argument`` start offset is less than timespan start
        offset.

        ..  container::: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)

            >>> timespan_1 < timespan_2
            True

            >>> timespan_2 < timespan_3
            False

        Returns true or false.
        """
        expr_start_offset, expr_stop_offset = \
            self._get_start_offset_and_maybe_stop_offset(argument)
        if expr_stop_offset is not None:
            if self._start_offset < expr_start_offset:
                return True
            elif (self._start_offset == expr_start_offset and
                self._stop_offset < expr_stop_offset):
                return True
            return False
        return self._start_offset < expr_start_offset

    def __or__(self, argument):
        """
        Logical OR of two timespans.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)
            >>> timespan_4 = abjad.Timespan(10, 20)

            >>> new_timespan = timespan_1 | timespan_2
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(12, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_1 | timespan_3
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_1 | timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_2 | timespan_3
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(2, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(5, 1),
                        stop_offset=abjad.Offset(12, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_2 | timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(5, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_3 | timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(2, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(10, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan list.
        """
        import abjad
        argument = self._get_timespan(argument)
        if (not self.intersects_timespan(argument) and
            not self.is_tangent_to_timespan(argument)):
            result = abjad.TimespanList([self, argument])
            result.sort()
            return result
        new_start_offset = min(self._start_offset, argument.start_offset)
        new_stop_offset = max(self._stop_offset, argument.stop_offset)
        timespan = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return abjad.TimespanList([timespan])

    def __sub__(self, argument):
        """
        Subtract ``argument`` from timespan.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)
            >>> timespan_4 = abjad.Timespan(10, 20)

            >>> timespan_1 - timespan_1
            TimespanList([])

            >>> timespan_1 - timespan_2
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))])

            >>> timespan_1 - timespan_3
            TimespanList([Timespan(start_offset=Offset(2, 1), stop_offset=Offset(10, 1))])

            >>> timespan_1 - timespan_4
            TimespanList([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))])

            >>> timespan_2 - timespan_1
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

            >>> timespan_2 - timespan_2
            TimespanList([])

            >>> timespan_2 - timespan_3
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(12, 1))])

            >>> timespan_2 - timespan_4
            TimespanList([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

            >>> timespan_3 - timespan_3
            TimespanList([])

            >>> timespan_3 - timespan_1
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))])

            >>> timespan_3 - timespan_2
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

            >>> timespan_3 - timespan_4
            TimespanList([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

            >>> timespan_4 - timespan_4
            TimespanList([])

            >>> timespan_4 - timespan_1
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

            >>> timespan_4 - timespan_2
            TimespanList([Timespan(start_offset=Offset(12, 1), stop_offset=Offset(20, 1))])

            >>> timespan_4 - timespan_3
            TimespanList([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

        Returns timespan list.
        """
        import abjad
        argument = self._get_timespan(argument)
        timespans = abjad.TimespanList()
        if not self.intersects_timespan(argument):
            timespans.append(copy.deepcopy(self))
        elif argument.trisects_timespan(self):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.contains_timespan_improperly(self):
            pass
        elif argument.overlaps_only_start_of_timespan(self):
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif argument.overlaps_only_stop_of_timespan(self):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif (argument.starts_when_timespan_starts(self) and
            argument.stops_before_timespan_stops(self)):
            new_start_offset = argument.stop_offset
            new_stop_offset = self._stop_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        elif (argument.stops_when_timespan_stops(self) and
            argument.starts_after_timespan_starts(self)):
            new_start_offset = self._start_offset
            new_stop_offset = argument.start_offset
            timespan = abjad.new(
                self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            timespans.append(timespan)
        else:
            raise ValueError(self, argument)
        return timespans

    def __xor__(self, argument):
        """
        Logical XOR of two timespans.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 12)
            >>> timespan_3 = abjad.Timespan(-2, 2)
            >>> timespan_4 = abjad.Timespan(10, 20)

            >>> new_timespan = timespan_1 ^ timespan_2
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(5, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(10, 1),
                        stop_offset=abjad.Offset(12, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_1 ^ timespan_3
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(0, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(2, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_1 ^ timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(10, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_2 ^ timespan_3
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(2, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(5, 1),
                        stop_offset=abjad.Offset(12, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_2 ^ timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(5, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(12, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

            >>> new_timespan = timespan_3 ^ timespan_4
            >>> abjad.f(new_timespan)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(-2, 1),
                        stop_offset=abjad.Offset(2, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(10, 1),
                        stop_offset=abjad.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan list.
        """
        import abjad
        argument = self._get_timespan(argument)
        if (not self.intersects_timespan(argument) or
            self.is_tangent_to_timespan(argument)):
            result = abjad.TimespanList()
            result.append(copy.deepcopy(self))
            result.append(copy.deepcopy(argument))
            result.sort()
            return result
        result = abjad.TimespanList()
        start_offsets = [self._start_offset, argument.start_offset]
        stop_offsets = [self._stop_offset, argument.stop_offset]
        start_offsets.sort()
        stop_offsets.sort()
        timespan_1 = abjad.new(
            self,
            start_offset=start_offsets[0],
            stop_offset=start_offsets[1],
            )
        timespan_2 = abjad.new(
            self,
            start_offset=stop_offsets[0],
            stop_offset=stop_offsets[1],
            )
        if timespan_1.is_well_formed:
            result.append(timespan_1)
        if timespan_2.is_well_formed:
            result.append(timespan_2)
        result.sort()
        return result

    ### PRIVATE METHODS ###

    def _as_postscript(
        self,
        postscript_x_offset,
        postscript_y_offset,
        postscript_scale,
        ):
        import abjad
        start = (float(self._start_offset) * postscript_scale)
        start -= postscript_x_offset
        stop = (float(self._stop_offset) * postscript_scale)
        stop -= postscript_x_offset
        ps = abjad.Postscript()
        ps = ps.moveto(start, postscript_y_offset)
        ps = ps.lineto(stop, postscript_y_offset)
        ps = ps.stroke()
        ps = ps.moveto(start, postscript_y_offset + 0.75)
        ps = ps.lineto(start, postscript_y_offset - 0.75)
        ps = ps.stroke()
        ps = ps.moveto(stop, postscript_y_offset + 0.75)
        ps = ps.lineto(stop, postscript_y_offset - 0.75)
        ps = ps.stroke()
        return ps

    def _can_fuse(self, argument):
        if isinstance(argument, type(self)):
            return self.intersects_timespan(argument) or \
                self.stops_when_timespan_starts(argument)
        return False

    @staticmethod
    def _get_offsets(argument):
        if isinstance(argument, Timespan):
            pass
        elif hasattr(argument, 'timespan'):
            argument = argument.timespan
        elif hasattr(argument, '_get_timespan'):
            argument = argument._get_timespan()
        else:
            raise ValueError(argument)
        return argument._start_offset, argument._stop_offset

    @staticmethod
    def _get_start_offset_and_maybe_stop_offset(argument):
        if isinstance(argument, Timespan):
            pass
        elif hasattr(argument, 'timespan'):
            argument = argument.timespan
        elif hasattr(argument, '_get_timespan'):
            argument = argument._get_timespan()
        start_offset = getattr(argument, 'start_offset', None)
        if start_offset is None:
            raise ValueError(argument)
        stop_offset = getattr(argument, 'stop_offset', None)
        return start_offset, stop_offset

    def _get_timespan(self, argument):
        import abjad
        if isinstance(argument, Timespan):
            start_offset, stop_offset = argument.offsets
        elif hasattr(argument, 'timespan'):
            start_offset, stop_offset = argument.timespan.offsets
        elif hasattr(argument, '_get_timespan'):
            start_offset, stop_offset = argument._get_timespan().offsets
        # TODO: remove this branch in favor of the _get_timespan above
        #elif hasattr(argument, 'timespan'):
        #    start_offset, stop_offset = argument.timespan().offsets
        else:
            raise ValueError(argument)
        return abjad.new(
            self,
            start_offset=start_offset,
            stop_offset=stop_offset,
            )

    @staticmethod
    def _implements_timespan_interface(timespan):
        if (
            getattr(timespan, 'start_offset', 'foo') != 'foo' and
            getattr(timespan, 'stop_offset', 'foo') != 'foo'
            ):
            return True
        if hasattr(timespan, '_get_timespan'):
            return True
        # TODO: remove this branch in favor of the _get_timespan above
        if hasattr(timespan, 'timespan'):
            return True
        if getattr(timespan, 'timespan', 'foo') != 'foo':
            return True
        return False

    def _initialize_offset(self, offset):
        import abjad
        if offset in (NegativeInfinity, Infinity):
            return offset
        return abjad.Offset(offset)

    ### PUBLIC METHODS ###

    def contains_timespan_improperly(self, timespan):
        """
        Is true when timespan contains ``timespan`` improperly.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 10)

            >>> timespan_1.contains_timespan_improperly(timespan_1)
            True
            >>> timespan_1.contains_timespan_improperly(timespan_2)
            True
            >>> timespan_2.contains_timespan_improperly(timespan_1)
            False
            >>> timespan_2.contains_timespan_improperly(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset <= expr_start_offset and
            expr_stop_offset <= self_stop_offset
            )

    def curtails_timespan(self, timespan):
        """
        Is true when timespan curtails ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 10)

            >>> timespan_1.curtails_timespan(timespan_1)
            False
            >>> timespan_1.curtails_timespan(timespan_2)
            False
            >>> timespan_2.curtails_timespan(timespan_1)
            True
            >>> timespan_2.curtails_timespan(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_start_offset and
            self_start_offset <= expr_stop_offset and
            expr_stop_offset <= self_stop_offset
            )

    def delays_timespan(self, timespan):
        """
        Is true when timespan delays ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)
            >>> timespan_3 = abjad.Timespan(10, 20)

            >>> timespan_1.delays_timespan(timespan_2)
            True
            >>> timespan_2.delays_timespan(timespan_3)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset <= expr_start_offset and
            expr_start_offset < self_stop_offset
            )

    def divide_by_ratio(self, ratio):
        """
        Divides timespan by ``ratio``.

        ..  container:: example

            >>> timespan = abjad.Timespan((1, 2), (3, 2))

            >>> for x in timespan.divide_by_ratio((1, 2, 1)):
            ...     x
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(3, 4))
            Timespan(start_offset=Offset(3, 4), stop_offset=Offset(5, 4))
            Timespan(start_offset=Offset(5, 4), stop_offset=Offset(3, 2))

        Returns tuple of newly constructed timespans.
        """
        import abjad
        if isinstance(ratio, int):
            ratio = ratio * (1, )
        ratio = abjad.Ratio(ratio)
        unit_duration = self.duration / sum(ratio.numbers)
        part_durations = [
            numerator * unit_duration for numerator in ratio.numbers
            ]
        start_offsets = abjad.mathtools.cumulative_sums(
            [self._start_offset] + part_durations,
            start=None,
            )
        offset_pairs = abjad.sequence(start_offsets).nwise()
        result = [type(self)(*offset_pair) for offset_pair in offset_pairs]
        return tuple(result)

    def get_overlap_with_timespan(self, timespan):
        """
        Gets duration of overlap with ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 15)
            >>> timespan_2 = abjad.Timespan(5, 10)
            >>> timespan_3 = abjad.Timespan(6, 6)
            >>> timespan_4 = abjad.Timespan(12, 22)

            >>> timespan_1.get_overlap_with_timespan(timespan_1)
            Duration(15, 1)

            >>> timespan_1.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

            >>> timespan_1.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

            >>> timespan_1.get_overlap_with_timespan(timespan_4)
            Duration(3, 1)

            >>> timespan_2.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

            >>> timespan_2.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

            >>> timespan_2.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

            >>> timespan_3.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

            >>> timespan_3.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

            >>> timespan_4.get_overlap_with_timespan(timespan_4)
            Duration(10, 1)

        Returns duration.
        """
        import abjad
        if self._implements_timespan_interface(timespan):
            result = abjad.Duration(
                sum(x.duration for x in self & timespan)
                )
            return result

    def happens_during_timespan(self, timespan):
        """
        Is true when timespan happens during ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 10)

            >>> timespan_1.happens_during_timespan(timespan_1)
            True
            >>> timespan_1.happens_during_timespan(timespan_2)
            False
            >>> timespan_2.happens_during_timespan(timespan_1)
            True
            >>> timespan_2.happens_during_timespan(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_stop_offset <= expr_stop_offset
            )

    def intersects_timespan(self, timespan):
        """
        Is true when timespan intersects ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)
            >>> timespan_3 = abjad.Timespan(10, 15)

            >>> timespan_1.intersects_timespan(timespan_1)
            True
            >>> timespan_1.intersects_timespan(timespan_2)
            True
            >>> timespan_1.intersects_timespan(timespan_3)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            (
                expr_start_offset <= self_start_offset and
                self_start_offset < expr_stop_offset
            ) or (
                self_start_offset <= expr_start_offset and
                expr_start_offset < self_stop_offset
                )
            )

    def is_congruent_to_timespan(self, timespan):
        """
        Is true when timespan is congruent to ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.is_congruent_to_timespan(timespan_1)
            True
            >>> timespan_1.is_congruent_to_timespan(timespan_2)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_1)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset == self_start_offset and
            expr_stop_offset == self_stop_offset
            )

    def is_tangent_to_timespan(self, timespan):
        """
        Is true when timespan is tangent to ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.is_tangent_to_timespan(timespan_1)
            False
            >>> timespan_1.is_tangent_to_timespan(timespan_2)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_1)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_stop_offset == expr_start_offset or
            expr_stop_offset == self_start_offset
            )

    def overlaps_all_of_timespan(self, timespan):
        """
        Is true when timespan overlaps all of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 6)
            >>> timespan_3 = abjad.Timespan(5, 10)

            >>> timespan_1.overlaps_all_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_all_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_all_of_timespan(timespan_3)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_stop_offset < self_stop_offset
            )

    def overlaps_only_start_of_timespan(self, timespan):
        """
        Is true when timespan overlaps only start of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(-5, 5)
            >>> timespan_3 = abjad.Timespan(4, 6)
            >>> timespan_4 = abjad.Timespan(5, 15)

            >>> timespan_1.overlaps_only_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_4)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_start_offset < self_stop_offset and
            self_stop_offset <= expr_stop_offset
            )

    def overlaps_only_stop_of_timespan(self, timespan):
        """
        Is true when timespan overlaps only stop of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(-5, 5)
            >>> timespan_3 = abjad.Timespan(4, 6)
            >>> timespan_4 = abjad.Timespan(5, 15)

            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_4)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_start_offset < expr_stop_offset and
            expr_stop_offset < self_stop_offset
            )

    def overlaps_start_of_timespan(self, timespan):
        """
        Is true when timespan overlaps start of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(-5, 5)
            >>> timespan_3 = abjad.Timespan(4, 6)
            >>> timespan_4 = abjad.Timespan(5, 15)

            >>> timespan_1.overlaps_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_start_of_timespan(timespan_4)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_start_offset and
            expr_start_offset < self_stop_offset
            )

    def overlaps_stop_of_timespan(self, timespan):
        """
        Is true when timespan overlaps start of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(-5, 5)
            >>> timespan_3 = abjad.Timespan(4, 6)
            >>> timespan_4 = abjad.Timespan(5, 15)

            >>> timespan_1.overlaps_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_4)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            self_start_offset < expr_stop_offset and
            expr_stop_offset < self_stop_offset
            )

    def reflect(self, axis=None):
        """
        Reflects timespan about ``axis``.

        ..  container:: example

            Reverse timespan about timespan axis:

            >>> abjad.Timespan(3, 6).reflect()
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(6, 1))

        ..  container:: example

            Reverse timespan about arbitrary axis:

            >>> abjad.Timespan(3, 6).reflect(axis=abjad.Offset(10))
            Timespan(start_offset=Offset(14, 1), stop_offset=Offset(17, 1))

        Returns new timespan.
        """
        if axis is None:
            axis = self.axis
        start_distance = self._start_offset - axis
        stop_distance = self._stop_offset - axis
        new_start_offset = axis - stop_distance
        new_stop_offset = axis - start_distance
        return self.set_offsets(new_start_offset, new_stop_offset)

    def round_offsets(self, multiplier, anchor=enums.Left, must_be_well_formed=True):
        """
        Rounds timespan offsets to multiple of ``multiplier``.

        ..  container:: example

            >>> timespan = abjad.Timespan((1, 5), (4, 5))

            >>> timespan.round_offsets(1)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(1, 1))

            >>> timespan.round_offsets(2)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=abjad.Right,
            ...     )
            Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=abjad.Right,
            ...     must_be_well_formed=False,
            ...     )
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(0, 1))

        Returns new timespan.
        """
        import abjad
        multiplier = abs(abjad.Multiplier(multiplier))
        assert 0 < multiplier
        new_start_offset = abjad.Offset(
            int(round(self._start_offset / multiplier)) * multiplier)
        new_stop_offset = abjad.Offset(
            int(round(self._stop_offset / multiplier)) * multiplier)
        if (new_start_offset == new_stop_offset) and must_be_well_formed:
            if anchor is enums.Left:
                new_stop_offset = new_stop_offset + multiplier
            else:
                new_start_offset = new_start_offset - multiplier
        result = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def scale(self, multiplier, anchor=enums.Left):
        """
        Scales timespan by ``multiplier``.

            >>> timespan = abjad.Timespan(3, 6)

        ..  container:: example

            Scale timespan relative to timespan start offset:

            >>> timespan.scale(abjad.Multiplier(2))
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(9, 1))

        ..  container:: example

            Scale timespan relative to timespan stop offset:

            >>> timespan.scale(abjad.Multiplier(2), anchor=abjad.Right)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(6, 1))

        Returns new timespan.
        """
        import abjad
        multiplier = abjad.Multiplier(multiplier)
        assert 0 < multiplier
        new_duration = multiplier * self.duration
        if anchor == enums.Left:
            new_start_offset = self._start_offset
            new_stop_offset = self._start_offset + new_duration
        elif anchor == enums.Right:
            new_stop_offset = self._stop_offset
            new_start_offset = self._stop_offset - new_duration
        else:
            message = 'unknown anchor direction: {!r}.'
            message = message.format(anchor)
            raise ValueError(message)
        result = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def set_duration(self, duration):
        """
        Sets timespan duration to ``duration``.

        ..  container:: example

            >>> timespan = abjad.Timespan((1, 2), (3, 2))

            >>> timespan.set_duration((3, 5))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(11, 10))

        Returns new timespan.
        """
        import abjad
        duration = abjad.Duration(duration)
        new_stop_offset = self._start_offset + duration
        return abjad.new(
            self,
            stop_offset=new_stop_offset,
            )

    def set_offsets(self, start_offset=None, stop_offset=None):
        """
        Sets timespan start offset to ``start_offset`` and
        stop offset to ``stop_offset``.

        ..  container:: example

            >>> timespan = abjad.Timespan((1, 2), (3, 2))

            >>> timespan.set_offsets(stop_offset=(7, 8))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(7, 8))

        Subtracts negative ``start_offset`` from existing stop offset:

        >>> timespan.set_offsets(start_offset=(-1, 2))
        Timespan(start_offset=Offset(1, 1), stop_offset=Offset(3, 2))

        Subtracts negative ``stop_offset`` from existing stop offset:

        >>> timespan.set_offsets(stop_offset=(-1, 2))
        Timespan(start_offset=Offset(1, 2), stop_offset=Offset(1, 1))

        Returns new timespan.
        """
        import abjad
        if start_offset is not None:
            start_offset = abjad.Offset(start_offset)

        if stop_offset is not None:
            stop_offset = abjad.Offset(stop_offset)
        if start_offset is not None and 0 <= start_offset:
            new_start_offset = start_offset
        elif start_offset is not None and start_offset < 0:
            new_start_offset = \
                self._stop_offset + abjad.Offset(start_offset)
        else:
            new_start_offset = self._start_offset
        if stop_offset is not None and 0 <= stop_offset:
            new_stop_offset = stop_offset
        elif stop_offset is not None and stop_offset < 0:
            new_stop_offset = \
                self._stop_offset + abjad.Offset(stop_offset)
        else:
            new_stop_offset = self._stop_offset
        result = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    # TODO: extend to self.split_at_offsets()
    def split_at_offset(self, offset):
        """
        Split into two parts when ``offset`` happens during timespan:

        ..  container:: example

            >>> timespan = abjad.Timespan(0, 5)

            >>> left, right = timespan.split_at_offset((2, 1))

            >>> left
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

            >>> right
            Timespan(start_offset=Offset(2, 1), stop_offset=Offset(5, 1))

            Otherwise return a copy of timespan:

            >>> timespan.split_at_offset((12, 1))[0]
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))

        Returns one or two newly constructed timespans.
        """
        import abjad
        offset = abjad.Offset(offset)
        result = abjad.TimespanList()
        if self._start_offset < offset < self._stop_offset:
            left = abjad.new(
                self,
                start_offset=self._start_offset,
                stop_offset=offset,
                )
            right = abjad.new(
                self,
                start_offset=offset,
                stop_offset=self._stop_offset,
                )
            result.append(left)
            result.append(right)
        else:
            result.append(abjad.new(self))
        return result

    def split_at_offsets(self, offsets):
        """
        Split into one or more parts when ``offsets`` happens during
        timespan:

        ..  container:: example

            >>> timespan = abjad.Timespan(0, 10)

            >>> result = timespan.split_at_offsets((1, 3, 7))
            >>> abjad.f(result)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(1, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(1, 1),
                        stop_offset=abjad.Offset(3, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(3, 1),
                        stop_offset=abjad.Offset(7, 1),
                        ),
                    abjad.Timespan(
                        start_offset=abjad.Offset(7, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    ]
                )

            Otherwise return a timespan list containing a copy of timespan:

            >>> result = timespan.split_at_offsets((-100,))
            >>> abjad.f(result)
            abjad.TimespanList(
                [
                    abjad.Timespan(
                        start_offset=abjad.Offset(0, 1),
                        stop_offset=abjad.Offset(10, 1),
                        ),
                    ]
                )

        Returns one or more newly constructed timespans.
        """
        import abjad
        offsets = [abjad.Offset(offset) for offset in offsets]
        offsets = [offset for offset in offsets
            if self._start_offset < offset < self._stop_offset]
        offsets = sorted(set(offsets))
        result = abjad.TimespanList()
        right = abjad.new(self)
        for offset in offsets:
            left, right = right.split_at_offset(offset)
            result.append(left)
        result.append(right)
        return result

    def starts_after_offset(self, offset):
        """
        Is true when timespan overlaps start of ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_after_offset((-5, 1))
            True
            >>> timespan_1.starts_after_offset((0, 1))
            False
            >>> timespan_1.starts_after_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return offset < self._start_offset

    def starts_after_timespan_starts(self, timespan):
        """
        Is true when timespan starts after ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.starts_after_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.starts_after_timespan_starts(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset < self_start_offset

    def starts_after_timespan_stops(self, timespan):
        """
        Is true when timespan starts after ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)
            >>> timespan_3 = abjad.Timespan(10, 20)
            >>> timespan_4 = abjad.Timespan(15, 25)

            >>> timespan_1.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_2.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_3.starts_after_timespan_stops(timespan_1)
            True
            >>> timespan_4.starts_after_timespan_stops(timespan_1)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_stop_offset <= self_start_offset

    def starts_at_offset(self, offset):
        """
        Is true when timespan starts at ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_at_offset((-5, 1))
            False
            >>> timespan_1.starts_at_offset((0, 1))
            True
            >>> timespan_1.starts_at_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._start_offset == offset

    def starts_at_or_after_offset(self, offset):
        """
        Is true when timespan starts at or after ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_at_or_after_offset((-5, 1))
            True
            >>> timespan_1.starts_at_or_after_offset((0, 1))
            True
            >>> timespan_1.starts_at_or_after_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return offset <= self._start_offset

    def starts_before_offset(self, offset):
        """
        Is true when timespan starts before ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_before_offset((-5, 1))
            False
            >>> timespan_1.starts_before_offset((0, 1))
            False
            >>> timespan_1.starts_before_offset((5, 1))
            True

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._start_offset < offset

    def starts_before_or_at_offset(self, offset):
        """
        Is true when timespan starts before or at ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_before_or_at_offset((-5, 1))
            False
            >>> timespan_1.starts_before_or_at_offset((0, 1))
            True
            >>> timespan_1.starts_before_or_at_offset((5, 1))
            True

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._start_offset <= offset

    def starts_before_timespan_starts(self, timespan):
        """
        Is true when timespan starts before ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_before_timespan_starts(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_before_timespan_starts(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset < expr_start_offset

    def starts_before_timespan_stops(self, timespan):
        """
        Is true when timespan starts before ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_1.starts_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset < expr_stop_offset

    def starts_during_timespan(self, timespan):
        """
        Is true when timespan starts during ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.starts_during_timespan(timespan_1)
            True
            >>> timespan_1.starts_during_timespan(timespan_2)
            False
            >>> timespan_2.starts_during_timespan(timespan_1)
            True
            >>> timespan_2.starts_during_timespan(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset <= self_start_offset and
            self_start_offset < expr_stop_offset
            )

    def starts_when_timespan_starts(self, timespan):
        """
        Is true when timespan starts when ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 15)

            >>> timespan_1.starts_when_timespan_starts(timespan_1)
            True
            >>> timespan_1.starts_when_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset == self_start_offset

    def starts_when_timespan_stops(self, timespan):
        """
        Is true when timespan starts when ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.starts_when_timespan_stops(timespan_1)
            False
            >>> timespan_1.starts_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_when_timespan_stops(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_start_offset == expr_stop_offset

    def stops_after_offset(self, offset):
        """
        Is true when timespan stops after ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.starts_after_offset((-5, 1))
            True
            >>> timespan_1.starts_after_offset((0, 1))
            False
            >>> timespan_1.starts_after_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return offset < self._stop_offset

    def stops_after_timespan_starts(self, timespan):
        """
        Is true when timespan stops when ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_1.stops_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_starts(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_start_offset < self_stop_offset

    def stops_after_timespan_stops(self, timespan):
        """
        Is true when timespan stops when ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_after_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_after_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_stops(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_stops(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return expr_stop_offset < self_stop_offset

    def stops_at_offset(self, offset):
        """
        Is true when timespan stops at ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.stops_at_offset((-5, 1))
            False
            >>> timespan_1.stops_at_offset((0, 1))
            False
            >>> timespan_1.stops_at_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._stop_offset == offset

    def stops_at_or_after_offset(self, offset):
        """
        Is true when timespan stops at or after ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.stops_at_or_after_offset((-5, 1))
            True
            >>> timespan_1.stops_at_or_after_offset((0, 1))
            True
            >>> timespan_1.stops_at_or_after_offset((5, 1))
            True

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return offset <= self._stop_offset

    def stops_before_offset(self, offset):
        """
        Is true when timespan stops before ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.stops_before_offset((-5, 1))
            False
            >>> timespan_1.stops_before_offset((0, 1))
            False
            >>> timespan_1.stops_before_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._stop_offset < offset

    def stops_before_or_at_offset(self, offset):
        """
        Is true when timespan stops before or at ``offset``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)

            >>> timespan_1.stops_before_or_at_offset((-5, 1))
            False
            >>> timespan_1.stops_before_or_at_offset((0, 1))
            False
            >>> timespan_1.stops_before_or_at_offset((5, 1))
            False

        Returns true or false.
        """
        import abjad
        offset = abjad.Offset(offset)
        return self._stop_offset <= offset

    def stops_before_timespan_starts(self, timespan):
        """
        Is true when timespan stops before ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset < expr_start_offset

    def stops_before_timespan_stops(self, timespan):
        """
        Is true when timespan stops before ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_stops(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset < expr_stop_offset

    def stops_during_timespan(self, timespan):
        """
        Is true when timespan stops during ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_during_timespan(timespan_1)
            True
            >>> timespan_1.stops_during_timespan(timespan_2)
            False
            >>> timespan_2.stops_during_timespan(timespan_1)
            False
            >>> timespan_2.stops_during_timespan(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_stop_offset and
            self_stop_offset <= expr_stop_offset
            )

    def stops_when_timespan_starts(self, timespan):
        """
        Is true when timespan stops when ``timespan`` starts.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_when_timespan_starts(timespan_2)
            True
            >>> timespan_2.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_starts(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset == expr_start_offset

    def stops_when_timespan_stops(self, timespan):
        """
        Is true when timespan stops when ``timespan`` stops.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(10, 20)

            >>> timespan_1.stops_when_timespan_stops(timespan_1)
            True
            >>> timespan_1.stops_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_2)
            True

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return self_stop_offset == expr_stop_offset

    def stretch(self, multiplier, anchor=None):
        """
        Stretches timespan by ``multiplier`` relative to ``anchor``.

        .. container:: example

            Stretch relative to timespan start offset:

            >>> abjad.Timespan(3, 10).stretch(abjad.Multiplier(2))
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(17, 1))

        .. container:: example

            Stretch relative to timespan stop offset:

            >>> abjad.Timespan(3, 10).stretch(
            ...     abjad.Multiplier(2),
            ...     abjad.Offset(10),
            ...     )
            Timespan(start_offset=Offset(-4, 1), stop_offset=Offset(10, 1))

        .. container:: example

            Stretch relative to offset prior to timespan:

            >>> abjad.Timespan(3, 10).stretch(
            ...     abjad.Multiplier(2),
            ...     abjad.Offset(0, 1),
            ...     )
            Timespan(start_offset=Offset(6, 1), stop_offset=Offset(20, 1))

        .. container:: example

            Stretch relative to offset after timespan:

            >>> abjad.Timespan(3, 10).stretch(
            ...     abjad.Multiplier(3),
            ...     abjad.Offset(12),
            ...     )
            Timespan(start_offset=Offset(-15, 1), stop_offset=Offset(6, 1))

        .. container:: example

            Stretch relative to offset that happens during timespan:

            >>> abjad.Timespan(3, 10).stretch(
            ...     abjad.Multiplier(2),
            ...     abjad.Offset(4),
            ...     )
            Timespan(start_offset=Offset(2, 1), stop_offset=Offset(16, 1))

        Returns newly emitted timespan.
        """
        import abjad
        multiplier = abjad.Multiplier(multiplier)
        assert 0 < multiplier
        if anchor is None:
            anchor = self._start_offset
        new_start_offset = (multiplier * (self._start_offset - anchor)) + anchor
        new_stop_offset = (multiplier * (self._stop_offset - anchor)) + anchor
        result = abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def translate(self, translation=None):
        """
        Translates timespan by ``translation``.

        ..  container:: example

            >>> timespan = abjad.Timespan(5, 10)

            >>> timespan.translate(2)
            Timespan(start_offset=Offset(7, 1), stop_offset=Offset(12, 1))

        Returns new timespan.
        """
        return self.translate_offsets(translation, translation)

    def translate_offsets(
        self,
        start_offset_translation=None,
        stop_offset_translation=None,
        ):
        """
        Translates timespan start offset by ``start_offset_translation`` and
        stop offset by ``stop_offset_translation``.

        ..  container:: example

            >>> timespan = abjad.Timespan((1, 2), (3, 2))

            >>> timespan.translate_offsets(start_offset_translation=(-1, 8))
            Timespan(start_offset=Offset(3, 8), stop_offset=Offset(3, 2))

        Returns new timespan.
        """
        import abjad
        start_offset_translation = start_offset_translation or 0
        stop_offset_translation = stop_offset_translation or 0
        start_offset_translation = abjad.Duration(start_offset_translation)
        stop_offset_translation = abjad.Duration(stop_offset_translation)
        new_start_offset = self._start_offset + start_offset_translation
        new_stop_offset = self._stop_offset + stop_offset_translation
        return abjad.new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )

    def trisects_timespan(self, timespan):
        """
        Is true when timespan trisects ``timespan``.

        ..  container:: example

            >>> timespan_1 = abjad.Timespan(0, 10)
            >>> timespan_2 = abjad.Timespan(5, 6)

            >>> timespan_1.trisects_timespan(timespan_1)
            False
            >>> timespan_1.trisects_timespan(timespan_2)
            False
            >>> timespan_2.trisects_timespan(timespan_1)
            True
            >>> timespan_2.trisects_timespan(timespan_2)
            False

        Returns true or false.
        """
        self_start_offset, self_stop_offset = self.offsets
        expr_start_offset, expr_stop_offset = self._get_offsets(timespan)
        return (
            expr_start_offset < self_start_offset and
            self_stop_offset < expr_stop_offset
            )

    ### PUBLIC PROPERTIES ###

    @property
    def axis(self):
        """
        Arithmetic mean of timespan start- and stop-offsets.

        ..  container:: example

            >>> abjad.Timespan(0, 10).axis
            Offset(5, 1)

        Returns offset.
        """
        return (self._start_offset + self._stop_offset) / 2

    @property
    def duration(self):
        """
        Duration of timespan.

        ..  container:: example

            >>> abjad.Timespan(0, 10).duration
            Duration(10, 1)

        Returns duration.
        """
        return self._stop_offset - self._start_offset

    @property
    def is_well_formed(self):
        """
        Is true when timespan start offset preceeds timespan stop offset.

        ..  container:: example

            >>> abjad.Timespan(0, 10).is_well_formed
            True

        Returns true or false.
        """
        return self._start_offset < self._stop_offset

    @property
    def offsets(self):
        """
        Timespan offsets.

        ..  container:: example

            >>> abjad.Timespan(0, 10).offsets
            (Offset(0, 1), Offset(10, 1))

        Returns offset pair.
        """
        return self._start_offset, self._stop_offset

    @property
    def start_offset(self):
        """
        Timespan start offset.

        ..  container:: example

            >>> abjad.Timespan(0, 10).start_offset
            Offset(0, 1)

        Returns offset.
        """
        return self._start_offset

    @property
    def stop_offset(self):
        """
        Timespan stop offset.

        ..  container:: example

            >>> abjad.Timespan(0, 10).stop_offset
            Offset(10, 1)

        Returns offset.
        """
        return self._stop_offset



import typing
from abjad.system.AbjadValueObject import AbjadValueObject


class LilyPondGrobInterface(AbjadValueObject):
    """
    LilyPond grob interface.

    ..  container:: example

        >>> interface = abjad.lilypondnames.LilyPondGrobInterface('beam-interface')
        >>> abjad.f(interface)
        LilyPondGrobInterface(name='beam-interface')

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        )

    _identity_map: typing.Dict[str, 'LilyPondGrobInterface'] = {}

    ### CONSTRUCTOR ###

    def __new__(class_, name='grob-interface'):
        if name in class_._identity_map:
            obj = class_._identity_map[name]
        else:
            obj = object.__new__(class_)
            class_._identity_map[name] = obj
        return obj

    ### INITIALIZER ###

    def __init__(self, name: str = 'grob-interface') -> None:
        from abjad.ly import interface_properties
        assert name in interface_properties
        self._name = name

    ### PUBLIC METHODS ###

    @staticmethod
    def list_all_interfaces() -> typing.Tuple['LilyPondGrobInterface', ...]:
        """
        Lists all interfaces.

        ..  container:: example

            >>> for grob_interface in abjad.lilypondnames.LilyPondGrobInterface.list_all_interfaces():
            ...     grob_interface
            ...
            LilyPondGrobInterface(name='accidental-interface')
            LilyPondGrobInterface(name='accidental-placement-interface')
            LilyPondGrobInterface(name='accidental-suggestion-interface')
            LilyPondGrobInterface(name='align-interface')
            LilyPondGrobInterface(name='ambitus-interface')
            LilyPondGrobInterface(name='arpeggio-interface')
            LilyPondGrobInterface(name='axis-group-interface')
            LilyPondGrobInterface(name='balloon-interface')
            LilyPondGrobInterface(name='bar-line-interface')
            LilyPondGrobInterface(name='bass-figure-alignment-interface')
            LilyPondGrobInterface(name='bass-figure-interface')
            LilyPondGrobInterface(name='beam-interface')
            LilyPondGrobInterface(name='bend-after-interface')
            LilyPondGrobInterface(name='break-alignable-interface')
            LilyPondGrobInterface(name='break-aligned-interface')
            LilyPondGrobInterface(name='break-alignment-interface')
            LilyPondGrobInterface(name='breathing-sign-interface')
            LilyPondGrobInterface(name='chord-name-interface')
            LilyPondGrobInterface(name='clef-interface')
            LilyPondGrobInterface(name='clef-modifier-interface')
            LilyPondGrobInterface(name='cluster-beacon-interface')
            LilyPondGrobInterface(name='cluster-interface')
            LilyPondGrobInterface(name='custos-interface')
            LilyPondGrobInterface(name='dot-column-interface')
            LilyPondGrobInterface(name='dots-interface')
            LilyPondGrobInterface(name='dynamic-interface')
            LilyPondGrobInterface(name='dynamic-line-spanner-interface')
            LilyPondGrobInterface(name='dynamic-text-interface')
            LilyPondGrobInterface(name='dynamic-text-spanner-interface')
            LilyPondGrobInterface(name='enclosing-bracket-interface')
            LilyPondGrobInterface(name='episema-interface')
            LilyPondGrobInterface(name='figured-bass-continuation-interface')
            LilyPondGrobInterface(name='finger-interface')
            LilyPondGrobInterface(name='fingering-column-interface')
            LilyPondGrobInterface(name='flag-interface')
            LilyPondGrobInterface(name='font-interface')
            LilyPondGrobInterface(name='footnote-interface')
            LilyPondGrobInterface(name='footnote-spanner-interface')
            LilyPondGrobInterface(name='fret-diagram-interface')
            LilyPondGrobInterface(name='glissando-interface')
            LilyPondGrobInterface(name='grace-spacing-interface')
            LilyPondGrobInterface(name='gregorian-ligature-interface')
            LilyPondGrobInterface(name='grid-line-interface')
            LilyPondGrobInterface(name='grid-point-interface')
            LilyPondGrobInterface(name='grob-interface')
            LilyPondGrobInterface(name='hairpin-interface')
            LilyPondGrobInterface(name='hara-kiri-group-spanner-interface')
            LilyPondGrobInterface(name='horizontal-bracket-interface')
            LilyPondGrobInterface(name='inline-accidental-interface')
            LilyPondGrobInterface(name='instrument-specific-markup-interface')
            LilyPondGrobInterface(name='item-interface')
            LilyPondGrobInterface(name='key-cancellation-interface')
            LilyPondGrobInterface(name='key-signature-interface')
            LilyPondGrobInterface(name='kievan-ligature-interface')
            LilyPondGrobInterface(name='ledger-line-spanner-interface')
            LilyPondGrobInterface(name='ledgered-interface')
            LilyPondGrobInterface(name='ligature-bracket-interface')
            LilyPondGrobInterface(name='ligature-head-interface')
            LilyPondGrobInterface(name='ligature-interface')
            LilyPondGrobInterface(name='line-interface')
            LilyPondGrobInterface(name='line-spanner-interface')
            LilyPondGrobInterface(name='lyric-extender-interface')
            LilyPondGrobInterface(name='lyric-hyphen-interface')
            LilyPondGrobInterface(name='lyric-interface')
            LilyPondGrobInterface(name='lyric-syllable-interface')
            LilyPondGrobInterface(name='mark-interface')
            LilyPondGrobInterface(name='measure-counter-interface')
            LilyPondGrobInterface(name='measure-grouping-interface')
            LilyPondGrobInterface(name='melody-spanner-interface')
            LilyPondGrobInterface(name='mensural-ligature-interface')
            LilyPondGrobInterface(name='metronome-mark-interface')
            LilyPondGrobInterface(name='multi-measure-interface')
            LilyPondGrobInterface(name='multi-measure-rest-interface')
            LilyPondGrobInterface(name='note-collision-interface')
            LilyPondGrobInterface(name='note-column-interface')
            LilyPondGrobInterface(name='note-head-interface')
            LilyPondGrobInterface(name='note-name-interface')
            LilyPondGrobInterface(name='note-spacing-interface')
            LilyPondGrobInterface(name='number-interface')
            LilyPondGrobInterface(name='only-prebreak-interface')
            LilyPondGrobInterface(name='ottava-bracket-interface')
            LilyPondGrobInterface(name='outside-staff-axis-group-interface')
            LilyPondGrobInterface(name='outside-staff-interface')
            LilyPondGrobInterface(name='paper-column-interface')
            LilyPondGrobInterface(name='parentheses-interface')
            LilyPondGrobInterface(name='percent-repeat-interface')
            LilyPondGrobInterface(name='percent-repeat-item-interface')
            LilyPondGrobInterface(name='piano-pedal-bracket-interface')
            LilyPondGrobInterface(name='piano-pedal-interface')
            LilyPondGrobInterface(name='piano-pedal-script-interface')
            LilyPondGrobInterface(name='pitched-trill-interface')
            LilyPondGrobInterface(name='pure-from-neighbor-interface')
            LilyPondGrobInterface(name='rest-collision-interface')
            LilyPondGrobInterface(name='rest-interface')
            LilyPondGrobInterface(name='rhythmic-grob-interface')
            LilyPondGrobInterface(name='rhythmic-head-interface')
            LilyPondGrobInterface(name='script-column-interface')
            LilyPondGrobInterface(name='script-interface')
            LilyPondGrobInterface(name='self-alignment-interface')
            LilyPondGrobInterface(name='semi-tie-column-interface')
            LilyPondGrobInterface(name='semi-tie-interface')
            LilyPondGrobInterface(name='separation-item-interface')
            LilyPondGrobInterface(name='side-position-interface')
            LilyPondGrobInterface(name='slur-interface')
            LilyPondGrobInterface(name='spaceable-grob-interface')
            LilyPondGrobInterface(name='spacing-interface')
            LilyPondGrobInterface(name='spacing-options-interface')
            LilyPondGrobInterface(name='spacing-spanner-interface')
            LilyPondGrobInterface(name='span-bar-interface')
            LilyPondGrobInterface(name='spanner-interface')
            LilyPondGrobInterface(name='staff-grouper-interface')
            LilyPondGrobInterface(name='staff-spacing-interface')
            LilyPondGrobInterface(name='staff-symbol-interface')
            LilyPondGrobInterface(name='staff-symbol-referencer-interface')
            LilyPondGrobInterface(name='stanza-number-interface')
            LilyPondGrobInterface(name='stem-interface')
            LilyPondGrobInterface(name='stem-tremolo-interface')
            LilyPondGrobInterface(name='string-number-interface')
            LilyPondGrobInterface(name='stroke-finger-interface')
            LilyPondGrobInterface(name='system-interface')
            LilyPondGrobInterface(name='system-start-delimiter-interface')
            LilyPondGrobInterface(name='system-start-text-interface')
            LilyPondGrobInterface(name='tab-note-head-interface')
            LilyPondGrobInterface(name='text-interface')
            LilyPondGrobInterface(name='text-script-interface')
            LilyPondGrobInterface(name='tie-column-interface')
            LilyPondGrobInterface(name='tie-interface')
            LilyPondGrobInterface(name='time-signature-interface')
            LilyPondGrobInterface(name='trill-pitch-accidental-interface')
            LilyPondGrobInterface(name='trill-spanner-interface')
            LilyPondGrobInterface(name='tuplet-bracket-interface')
            LilyPondGrobInterface(name='tuplet-number-interface')
            LilyPondGrobInterface(name='unbreakable-spanner-interface')
            LilyPondGrobInterface(name='vaticana-ligature-interface')
            LilyPondGrobInterface(name='volta-bracket-interface')
            LilyPondGrobInterface(name='volta-interface')

        """
        from abjad.ly import interface_properties
        return tuple(
            LilyPondGrobInterface(_)
            for _ in sorted(interface_properties)
            )

    ### PUBLIC PROPERTIES ###

    @property
    def name(self) -> str:
        """
        Gets name of LilyPond grob interface.

        ..  container:: example

            >>> interface = abjad.lilypondnames.LilyPondGrobInterface('beam-interface')
            >>> interface.name
            'beam-interface'

        """
        return self._name

    @property
    def property_names(self) -> typing.Tuple[str, ...]:
        """
        Gets property names of LilyPond grob interface.

        ..  container:: example

            >>> interface = abjad.lilypondnames.LilyPondGrobInterface('beam-interface')
            >>> for property_name in interface.property_names:
            ...     property_name
            ...
            'X-positions'
            'annotation'
            'auto-knee-gap'
            'beam-thickness'
            'beamed-stem-shorten'
            'beaming'
            'break-overshoot'
            'clip-edges'
            'collision-interfaces'
            'collision-voice-only'
            'concaveness'
            'damping'
            'details'
            'direction'
            'gap'
            'gap-count'
            'grow-direction'
            'inspect-quants'
            'knee'
            'length-fraction'
            'neutral-direction'
            'positions'
            'skip-quanting'

        """
        from abjad.ly import interface_properties
        names = interface_properties[self.name]
        assert isinstance(names, list), repr(names)
        assert all(isinstance(_, str) for _ in names), repr(names)
        return tuple(names)



import typing
from abjad.system.AbjadObject import AbjadObject
from .FormatSpecification import FormatSpecification


class Tag(AbjadObject):
    """
    Tag.

    ..  container:: example

        >>> abjad.Tag('YELLOW')
        Tag('YELLOW')

        >>> abjad.Tag('YELLOW:RED')
        Tag('YELLOW:RED')

        Removes duplicate words at initialization:

        >>> abjad.Tag('YELLOW:RED:RED')
        Tag('YELLOW:RED')

    ..  container:: example

        Initializes from other tag:

        >>> abjad.Tag(abjad.Tag('YELLOW'))
        Tag('YELLOW')

    ..  container:: example

        Raises exception on multiple only-edition tags:

        >>> abjad.Tag('+SEGMENT:+PARTS')
        Traceback (most recent call last):
            ...
        Exception: at most one only-edition tag: ['+SEGMENT', '+PARTS'].

    ..  container:: example

        Raises exception on mixed only-edition / not-edition tags:

        >>> abjad.Tag('+SEGMENT:-PARTS')
        Traceback (most recent call last):
            ...
        Exception: only-edition and not-edition forbidden in same tag:
        <BLANKLINE>
        ['+SEGMENT'] / ['-PARTS']

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_string',
        '_words',
        )

    ### INITIALIZER ###

    def __init__(self, string: typing.Union[str, 'Tag'] = None) -> None:
        if isinstance(string, Tag):
            string = str(string)
        if string is not None:
            assert not string.startswith(':'), repr(string)
            words = string.split(':')
        else:
            words = []
        assert isinstance(words, list), repr(words)
        words_: typing.List[str] = []
        for word in words:
            if word not in words_:
                words_.append(word)
        only_edition_tags, not_edition_tags = [], []
        for word_ in words_:
            if word_.startswith('+'):
                only_edition_tags.append(word_)
            if word_.startswith('-'):
                not_edition_tags.append(word_)
        if 1 < len(only_edition_tags):
            message = f'at most one only-edition tag: {only_edition_tags!r}.'
            raise Exception(message)
        if only_edition_tags and not_edition_tags:
            message = 'only-edition and not-edition forbidden in same tag:\n\n'
            message += f'  {only_edition_tags} / {not_edition_tags}'
            raise Exception(message)
        self._words = words_
        if bool(string):
            string = ':'.join(words_)
            assert not string.startswith(':'), repr(string)
        else:
            string = None
        self._string = string

    ### SPECIAL METHODS ###

    def __bool__(self):
        """
        Is true when tag has words.

        ..  container:: example

            >>> bool(abjad.Tag())
            False

            >>> bool(abjad.Tag('+PARTS'))
            True

        """
        return bool(str(self))

    def __contains__(self, argument) -> bool:
        """
        Is true when ``argument`` is word in tag.

        ..  container:: example

            >>> tag = abjad.Tag('-PARTS')
            >>> tag = tag.append(abjad.tags.DEFAULT_CLEF)

            >>> 'PARTS' in tag
            False

            >>> '-PARTS' in tag
            True

            >>> abjad.tags.DEFAULT_CLEF in tag
            True

        """
        return argument in self.words

    def __eq__(self, argument):
        """
        Is true when ``argument`` is tag with same string representation.

        ..  container:: example

            >>> tag_1 = abjad.Tag()
            >>> tag_2 = abjad.Tag()
            >>> tag_3 = abjad.Tag('+PARTS')

            >>> tag_1 == tag_1
            True
            >>> tag_1 == tag_2
            True
            >>> tag_1 == tag_3
            False

            >>> tag_2 == tag_1
            True
            >>> tag_2 == tag_2
            True
            >>> tag_2 == tag_3
            False

            >>> tag_3 == tag_1
            False
            >>> tag_3 == tag_2
            False
            >>> tag_3 == tag_3
            True

        """
        if isinstance(argument, Tag):
            return str(self) == str(argument)
        return False

    def __hash__(self):
        """
        Hashes tag.
        """
        return super().__hash__()

    def __iter__(self):
        """
        Iterates words in tag.

        ..  container:: example

            >>> tag = abjad.Tag('-PARTS:-SCORE:DEFAULT_CLEF')
            >>> for word  in tag:
            ...     word
            ...
            '-PARTS'
            '-SCORE'
            'DEFAULT_CLEF'

        """
        return iter(self.words)

    def __str__(self):
        """
        Changes tag to string.

        ..  container:: example

            >>> str(abjad.Tag())
            ''

            >>> str(abjad.Tag('-PARTS:-SCORE:DEFAULT_CLEF'))
            '-PARTS:-SCORE:DEFAULT_CLEF'

        """
        return self.string or ''

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = []
        if self.string is not None:
            values.append(self.string)
        return FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=values,
            storage_format_is_indented=False,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def string(self) -> typing.Optional[str]:
        """
        Gets string.

        ..  container:: example

            >>> abjad.Tag().string is None
            True

            >>> abjad.Tag('-PARTS:DEFAULT_CLEF').string
            '-PARTS:DEFAULT_CLEF'

        """
        return self._string

    @property
    def words(self) -> typing.List[str]:
        """
        Gets words.

        ..  container:: example

            >>> abjad.Tag('-PARTS:DEFAULT_CLEF').words
            ['-PARTS', 'DEFAULT_CLEF']

        """
        return list(self._words)

    ### PUBLIC METHODS ###

    def append(self, word: str) -> 'Tag':
        """
        Appends ``word`` to tag.
        
        ..  container:: example

            >>> abjad.Tag('-PARTS').append(abjad.tags.DEFAULT_CLEF)
            Tag('-PARTS:DEFAULT_CLEF')

        """
        if not bool(word):
            return Tag(self)
        if isinstance(word, Tag):
            word = str(word)
        assert isinstance(word, str), repr(word)
        assert word != '', repr(word)
        words_ = self.words
        words_.append(word)
        return Tag.from_words(words_)

    def editions(self) -> typing.List['Tag']:
        """
        Gets edition tags in tag.

        ..  container:: example

            >>> abjad.Tag('FOO').editions()
            []

            >>> abjad.Tag('+SEGMENT').only_edition()
            Tag('+SEGMENT')

            >>> abjad.Tag('+SEGMENT:FOO').only_edition()
            Tag('+SEGMENT')

            >>> abjad.Tag('-SEGMENT').editions()
            [Tag('-SEGMENT')]

            >>> abjad.Tag('-SEGMENT:FOO').editions()
            [Tag('-SEGMENT')]

            >>> abjad.Tag('-SEGMENT:-PARTS').editions()
            [Tag('-SEGMENT'), Tag('-PARTS')]

        """
        result = []
        for word in self:
            if word.startswith('+') or word.startswith('-'):
                result.append(Tag(word))
        return result

    def extend(self, words: typing.List[str]) -> 'Tag':
        """
        Extends tag with ``words``.

        ..  container:: example

            >>> tag = abjad.Tag('-PARTS')
            >>> tag.extend(['-SCORE', abjad.tags.DEFAULT_CLEF])
            Tag('-PARTS:-SCORE:DEFAULT_CLEF')

        """
        assert isinstance(words, list), repr(words)
        tag = self
        for word in words:
            tag = tag.append(word)
        return tag

    @staticmethod
    def from_words(words: typing.List[str]) -> 'Tag':
        """
        Makes tag from ``words``.
        """
        assert isinstance(words, list), repr(words)
        words_ = []
        for word in words:
            if not bool(word):
                continue
            word = str(word)
            words_.append(word)
        string = ':'.join(words_)
        return Tag(string)

    def has_persistence_tag(self) -> bool:
        """
        Is true when tag has persistence tag.

        ..  container:: example

            >>> abjad.Tag('FOO').has_persistence_tag()
            False

            >>> abjad.Tag('FOO:DEFAULT_CLEF').has_persistence_tag()
            True

            >>> abjad.Tag('DEFAULT_CLEF').has_persistence_tag()
            True

        """
        from abjad.system.Tags import Tags
        tags = Tags().persistent_indicator_tags()
        for word in self:
            if word in tags:
                return True
        return False

    def invert_edition_tags(self) -> 'Tag':
        """
        Inverts edition tags in tag.

        ..  container:: example

            >>> abjad.Tag('FOO').invert_edition_tags()
            Tag('FOO')

            >>> abjad.Tag('FOO:-PARTS').invert_edition_tags()
            Tag('FOO:+PARTS')

            >>> abjad.Tag('FOO:+PARTS').invert_edition_tags()
            Tag('FOO:-PARTS')

        """
        words = []
        for word in self.words:
            if word.startswith('+'):
                word_ = '-' + word[1:]
            elif word.startswith('-'):
                word_ = '+' + word[1:]
            else:
                word_ = word
            words.append(word_)
        tag = Tag.from_words(words)
        return tag

    def not_editions(self) -> typing.List['Tag']:
        """
        Gets not-edition tags in tag.

        ..  container:: example

            >>> abjad.Tag('FOO').not_editions()
            []

            >>> abjad.Tag('-SEGMENT').not_editions()
            [Tag('-SEGMENT')]

            >>> abjad.Tag('-SEGMENT:FOO').not_editions()
            [Tag('-SEGMENT')]

            >>> abjad.Tag('-SEGMENT:-PARTS').not_editions()
            [Tag('-SEGMENT'), Tag('-PARTS')]

        """
        result = []
        for word in self:
            if word.startswith('-'):
                result.append(Tag(word))
        return result

    def only_edition(self) -> typing.Optional['Tag']:
        """
        Gets only-edition tag in tag.

        ..  container:: example

            >>> abjad.Tag('FOO').only_edition() is None
            True

            >>> abjad.Tag('+SEGMENT').only_edition()
            Tag('+SEGMENT')

            >>> abjad.Tag('+SEGMENT:FOO').only_edition()
            Tag('+SEGMENT')

        """
        for word in self:
            if word.startswith('+'):
                return Tag(word)
        else:
            return None

    def prepend(self, word: str) -> 'Tag':
        """
        Prepends ``word`` to tag.
        
        ..  container:: example

            >>> abjad.Tag('-PARTS').prepend(abjad.tags.DEFAULT_CLEF)
            Tag('DEFAULT_CLEF:-PARTS')

        """
        if not bool(word):
            return Tag(self)
        if isinstance(word, Tag):
            word = str(word)
        assert isinstance(word, str), repr(word)
        assert word != '', repr(word)
        words_ = self.words
        words_.insert(0, word)
        return Tag.from_words(words_)



import typing
from abjad.lilypondnames.LilyPondGrobOverride import LilyPondGrobOverride
from abjad.pitch.NamedInterval import NamedInterval
from abjad.pitch.NamedPitch import NamedPitch
from abjad.scheme import Scheme
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.top.override import override
from .Spanner import Spanner


class TrillSpanner(Spanner):
    r"""
    Trill spanner.

    ..  container:: example

        Attaches unpitched trill spanner to all notes in staff:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> trill = abjad.TrillSpanner()
        >>> abjad.attach(trill, staff[:])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                \startTrillSpan
                d'8
                e'8
                f'8
                \stopTrillSpan
            }

    ..  container:: example

        Attaches pitched trill spanner to all notes in staff:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> trill = abjad.TrillSpanner(pitch=abjad.NamedPitch("cs'"))
        >>> abjad.attach(trill, staff[:])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \pitchedTrill
                c'8
                \startTrillSpan cs'
                d'8
                e'8
                f'8
                \stopTrillSpan
            }

    ..  container:: example

        REGRESSION. Pitched trill spanner must appear after markup to avoid
        hiding markup in graphic output:

        >>> staff = abjad.Staff("c'4 d' e' r")
        >>> markup = abjad.Markup('Allegro', direction=abjad.Up)
        >>> abjad.attach(markup, staff[0])
        >>> trill_spanner = abjad.TrillSpanner(pitch='Db4')
        >>> abjad.attach(trill_spanner, staff[:])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \pitchedTrill
                c'4
                ^ \markup { Allegro }
                \startTrillSpan df'
                d'4
                e'4
                r4
                \stopTrillSpan
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_interval',
        '_is_harmonic',
        '_left_broken',
        '_pitch',
        '_right_broken',
        )

    _start_command = r'\startTrillSpan'

    _stop_command = r'\stopTrillSpan'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        interval: typing.Union[str, NamedInterval] = None,
        is_harmonic: bool = None,
        left_broken: bool = None,
        pitch: typing.Union[str, NamedPitch] = None,
        right_broken: bool = None,
        ) -> None:
        Spanner.__init__(self)
        if interval is not None and pitch is not None:
            message = 'only pitch or interval, not both:'
            message += f' {interval!r} + {pitch!r}.'
            raise Exception(message)
        if interval is not None:
            interval = NamedInterval(interval)
        self._interval = interval
        if is_harmonic is not None:
            is_harmonic = bool(is_harmonic)
        self._is_harmonic = is_harmonic
        if left_broken is not None:
            left_broken = bool(left_broken)
        self._left_broken = left_broken
        if pitch is not None:
            pitch = NamedPitch(pitch)
        self._pitch = pitch
        if right_broken is not None:
            right_broken = bool(right_broken)
        self._right_broken = right_broken

    ### PRIVATE METHODS ###

    def _copy_keywords(self, new):
        self._is_harmonic = self.is_harmonic
        new._pitch = self.pitch

    def _get_lilypond_format_bundle(self, leaf):
        bundle = LilyPondFormatBundle()
        if len(self) == 1 and self._left_broken:
            strings = [self._stop_command_string()]
            strings = self._tag_show(strings)
            bundle.after.spanner_stops.extend(strings)
            return bundle
        # important: pitch trill must start AFTER markup
        if leaf is self[0]:
            if self.pitch is not None or self.interval is not None:
                if self.is_harmonic:
                    string = '(lambda (grob) (grob-interpret-markup grob'
                    string += r' #{ \markup \musicglyph #"noteheads.s0harmonic" #}))'
                    scheme = Scheme(string, verbatim=True)
                    override = LilyPondGrobOverride(
                        grob_name='TrillPitchHead',
                        property_path=('stencil',),
                        value=scheme,
                        )
                    string = override.tweak_string(grob=True)
                    bundle.after.trill_spanner_starts.append(string)
                strings = [r'\pitchedTrill']
                if self._left_broken:
                    strings = self._tag_hide(strings)
                bundle.opening.spanners.extend(strings)
            if self.pitch is not None:
                pitch_string = str(self.pitch)
            elif self.interval is not None:
                pitch = leaf.written_pitch + self.interval
                pitch_string = str(pitch)
            else:
                pitch_string = None
            strings = self._tweaked_start_command_strings()
            if pitch_string:
                strings[-1] += ' ' + pitch_string
            if self._left_broken:
                strings = self._tag_hide(strings)
            bundle.after.trill_spanner_starts.extend(strings)
        if leaf is self[-1]:
            if 1 < len(self):
                strings = [self._stop_command_string()]
                if self._right_broken:
                    strings = self._tag_hide(strings)
                bundle.after.spanner_stops.extend(strings)
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def cross_segment_examples(self):
        r"""
        Cross-segment examples.

        ..  container:: example

            Cross-segment example #1 (one-to-one):

            >>> segment_1 = abjad.Voice("c'4 d' e' f'", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(right_broken=True)
            >>> abjad.attach(trill_spanner, segment_1[-1:])
            >>> abjad.show(segment_1, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_1, strict=50)
                \context Voice = "MainVoice"
                {
                    c'4
                    d'4
                    e'4
                    f'4
                    \startTrillSpan
                }

            >>> segment_2 = abjad.Voice("g'4 f'2 r4", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(left_broken=True)
            >>> abjad.attach(trill_spanner, segment_2[:1])
            >>> abjad.show(segment_2, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_2, strict=50)
                \context Voice = "MainVoice"
                {
                    g'4
                %@% \stopTrillSpan                                %! SHOW_TO_JOIN_BROKEN_SPANNERS
                    f'2
                    r4
                }

            >>> container = abjad.Container([segment_1, segment_2])
            >>> text = format(container, 'lilypond')
            >>> text = abjad.LilyPondFormatManager.left_shift_tags(text, 50)
            >>> job = abjad.Job.join_broken_spanners(text)
            >>> text = job()
            >>> lines = text.split('\n')
            >>> lilypond_file = abjad.LilyPondFile.new(lines)
            >>> abjad.show(lilypond_file, strict=50) # doctest: +SKIP

            ..  docs::

                >>> print(text)
                {
                    \context Voice = "MainVoice"
                    {
                        c'4
                        d'4
                        e'4
                        f'4
                        \startTrillSpan
                    }
                    \context Voice = "MainVoice"
                    {
                        g'4
                        \stopTrillSpan                            %! SHOW_TO_JOIN_BROKEN_SPANNERS %@%
                        f'2
                        r4
                    }
                }

        ..  container:: example

            Cross-segment example #2 (one-to-many):

            >>> segment_1 = abjad.Voice("c'4 d' e' f'", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(right_broken=True)
            >>> abjad.attach(trill_spanner, segment_1[-1:])
            >>> abjad.show(segment_1, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_1, strict=50)
                \context Voice = "MainVoice"
                {
                    c'4
                    d'4
                    e'4
                    f'4
                    \startTrillSpan
                }

            >>> segment_2 = abjad.Voice("g'4 f'2 r4", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(left_broken=True)
            >>> abjad.attach(trill_spanner, segment_2[:])
            >>> abjad.show(segment_2, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_2, strict=50)
                \context Voice = "MainVoice"
                {
                    g'4
                    \startTrillSpan                               %! HIDE_TO_JOIN_BROKEN_SPANNERS
                    f'2
                    r4
                    \stopTrillSpan
                }

            >>> container = abjad.Container([segment_1, segment_2])
            >>> text = format(container, 'lilypond')
            >>> text = abjad.LilyPondFormatManager.left_shift_tags(text, 50)
            >>> job = abjad.Job.join_broken_spanners(text)
            >>> text = job()
            >>> lines = text.split('\n')
            >>> lilypond_file = abjad.LilyPondFile.new(lines)
            >>> abjad.show(lilypond_file, strict=50) # doctest: +SKIP

            ..  docs::

                >>> print(text)
                {
                    \context Voice = "MainVoice"
                    {
                        c'4
                        d'4
                        e'4
                        f'4
                        \startTrillSpan
                    }
                    \context Voice = "MainVoice"
                    {
                        g'4
                    %%% \startTrillSpan                           %! HIDE_TO_JOIN_BROKEN_SPANNERS
                        f'2
                        r4
                        \stopTrillSpan
                    }
                }

        ..  container:: example

            Cross-segment example #3 (many-to-one):

            >>> segment_1 = abjad.Voice("c'4 d' e' f'", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(right_broken=True)
            >>> abjad.attach(trill_spanner, segment_1[:])
            >>> abjad.show(segment_1, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_1, strict=50)
                \context Voice = "MainVoice"
                {
                    c'4
                    \startTrillSpan
                    d'4
                    e'4
                    f'4
                    \stopTrillSpan                                %! HIDE_TO_JOIN_BROKEN_SPANNERS
                }

            >>> segment_2 = abjad.Voice("g'4 f'2 r4", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(left_broken=True)
            >>> abjad.attach(trill_spanner, segment_2[:1])
            >>> abjad.show(segment_2, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_2, strict=50)
                \context Voice = "MainVoice"
                {
                    g'4
                %@% \stopTrillSpan                                %! SHOW_TO_JOIN_BROKEN_SPANNERS
                    f'2
                    r4
                }

            >>> container = abjad.Container([segment_1, segment_2])
            >>> text = format(container, 'lilypond')
            >>> text = abjad.LilyPondFormatManager.left_shift_tags(text, 50)
            >>> job = abjad.Job.join_broken_spanners(text)
            >>> text = job()
            >>> lines = text.split('\n')
            >>> lilypond_file = abjad.LilyPondFile.new(lines)
            >>> abjad.show(lilypond_file, strict=50) # doctest: +SKIP

            ..  docs::

                >>> print(text)
                {
                    \context Voice = "MainVoice"
                    {
                        c'4
                        \startTrillSpan
                        d'4
                        e'4
                        f'4
                    %%% \stopTrillSpan                            %! HIDE_TO_JOIN_BROKEN_SPANNERS
                    }
                    \context Voice = "MainVoice"
                    {
                        g'4
                        \stopTrillSpan                            %! SHOW_TO_JOIN_BROKEN_SPANNERS %@%
                        f'2
                        r4
                    }
                }

        ..  container:: example

            Cross-segment example #4 (many-to-many):

            >>> segment_1 = abjad.Voice("c'4 d' e' f'", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(right_broken=True)
            >>> abjad.attach(trill_spanner, segment_1[:])
            >>> abjad.show(segment_1, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_1, strict=50)
                \context Voice = "MainVoice"
                {
                    c'4
                    \startTrillSpan
                    d'4
                    e'4
                    f'4
                    \stopTrillSpan                                %! HIDE_TO_JOIN_BROKEN_SPANNERS
                }

            >>> segment_2 = abjad.Voice("g'4 f'2 r4", name='MainVoice')
            >>> trill_spanner = abjad.TrillSpanner(left_broken=True)
            >>> abjad.attach(trill_spanner, segment_2[:])
            >>> abjad.show(segment_2, strict=50) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(segment_2, strict=50)
                \context Voice = "MainVoice"
                {
                    g'4
                    \startTrillSpan                               %! HIDE_TO_JOIN_BROKEN_SPANNERS
                    f'2
                    r4
                    \stopTrillSpan
                }

            >>> container = abjad.Container([segment_1, segment_2])
            >>> text = format(container, 'lilypond')
            >>> text = abjad.LilyPondFormatManager.left_shift_tags(text, 50)
            >>> job = abjad.Job.join_broken_spanners(text)
            >>> text = job()
            >>> lines = text.split('\n')
            >>> lilypond_file = abjad.LilyPondFile.new(lines)
            >>> abjad.show(lilypond_file, strict=50) # doctest: +SKIP

            ..  docs::

                >>> print(text)
                {
                    \context Voice = "MainVoice"
                    {
                        c'4
                        \startTrillSpan
                        d'4
                        e'4
                        f'4
                    %%% \stopTrillSpan                            %! HIDE_TO_JOIN_BROKEN_SPANNERS
                    }
                    \context Voice = "MainVoice"
                    {
                        g'4
                    %%% \startTrillSpan                           %! HIDE_TO_JOIN_BROKEN_SPANNERS
                        f'2
                        r4
                        \stopTrillSpan
                    }
                }

        """
        pass

    @property
    def interval(self) -> typing.Optional[NamedInterval]:
        r"""
        Gets interval of trill.

        ..  container:: example

            Attaches semitone trill:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> trill = abjad.TrillSpanner(
            ...     interval=abjad.NamedInterval('m2'),
            ...     )
            >>> abjad.attach(trill, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \pitchedTrill
                    c'8
                    \startTrillSpan df'
                    d'8
                    e'8
                    f'8
                    \stopTrillSpan
                }

        ..  container:: example

            Attaches whole tone trill:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> trill = abjad.TrillSpanner(
            ...     interval=abjad.NamedInterval('M2'),
            ...     )
            >>> abjad.attach(trill, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \pitchedTrill
                    c'8
                    \startTrillSpan d'
                    d'8
                    e'8
                    f'8
                    \stopTrillSpan
                }

        Defaults to none.

        Set to interval or none.

        Returns interval or none.
        """
        return self._interval

    @property
    def is_harmonic(self) -> typing.Optional[bool]:
        r"""
        Is true when trill pitch note-head is harmonic.

        ..  container:: example

            Attaches harmonic trill:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> trill = abjad.TrillSpanner(
            ...     is_harmonic=True,
            ...     pitch=abjad.NamedPitch("d'"),
            ...     )
            >>> abjad.attach(trill, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \pitchedTrill
                    c'8
                    - \tweak TrillPitchHead.stencil #(lambda (grob) (grob-interpret-markup grob #{ \markup \musicglyph #"noteheads.s0harmonic" #}))
                    \startTrillSpan d'
                    d'8
                    e'8
                    f'8
                    \stopTrillSpan
                }

        Defaults to false.

        Set to true or false.

        Returns true or false.
        """
        return self._is_harmonic

    @property
    def left_broken(self) -> typing.Optional[bool]:
        """
        Is true when spanner is left-broken.
        """
        return self._left_broken

    @property
    def pitch(self) -> typing.Optional[NamedPitch]:
        r"""
        Gets pitch.

        ..  container:: example

            Returns pitch when trill spanner is pitched:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> pitch = abjad.NamedPitch('C#4')
            >>> trill = abjad.TrillSpanner(pitch=pitch)
            >>> abjad.attach(trill, staff[:2])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \pitchedTrill
                    c'8
                    \startTrillSpan cs'
                    d'8
                    \stopTrillSpan
                    e'8
                    f'8
                }

            >>> trill.pitch
            NamedPitch("cs'")

        ..  container:: example

            Returns none when trill spanner is unpitched:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> trill = abjad.TrillSpanner()
            >>> abjad.attach(trill, staff[:2])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    \startTrillSpan
                    d'8
                    \stopTrillSpan
                    e'8
                    f'8
                }

            >>> trill.pitch is None
            True

        Formats LilyPond ``\pitchedTrill`` command on first leaf in spanner.

        Defaults to none.

        Set to named pitch or none.

        Returns named pitch or none.
        """
        return self._pitch

    @property
    def right_broken(self) -> typing.Optional[bool]:
        """
        Is true when spanner is right-broken.
        """
        return self._right_broken

    @property
    def written_pitch(self) -> typing.Optional[NamedPitch]:
        r"""
        Gets written pitch of trill spanner.

        ..  container:: example

            Returns pitch when trill spanner is pitched:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> pitch = abjad.NamedPitch('C#4')
            >>> trill = abjad.TrillSpanner(pitch=pitch)
            >>> abjad.attach(trill, staff[:2])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \pitchedTrill
                    c'8
                    \startTrillSpan cs'
                    d'8
                    \stopTrillSpan
                    e'8
                    f'8
                }

            >>> trill.written_pitch
            NamedPitch("cs'")

        ..  container:: example

            Returns none when trill spanner is unpitched:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> trill = abjad.TrillSpanner()
            >>> abjad.attach(trill, staff[:2])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    \startTrillSpan
                    d'8
                    \stopTrillSpan
                    e'8
                    f'8
                }

            >>> trill.written_pitch is None
            True

        Alias defined equal to ``pitch`` of trill spanner.

        Defaults to none.

        Property can not be set.

        Returns named pitch or none.
        """
        return self.pitch



import typing
from abjad import enums
from abjad.utilities.String import String
from .Spanner import Spanner


class PhrasingSlur(Spanner):
    r"""
    Phrasing slur.

    ..  container:: example

        Spans four notes:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> slur = abjad.PhrasingSlur()
        >>> abjad.attach(slur, staff[:])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                \(
                d'8
                e'8
                f'8
                \)
            }

    ..  container:: example

        Requires at least two leaves:

        >>> staff = abjad.Staff("c'8 d' e' f'")
        >>> phrasing_slur = abjad.PhrasingSlur()
        >>> abjad.attach(phrasing_slur, staff[:1])
        Traceback (most recent call last):
            ...
        Exception: PhrasingSlur()._attachment_test_all():
          Requires at least two leaves.
          Not just Note("c'8").

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    _start_command = r'\('

    _stop_command = r'\)'

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        direction: typing.Union[str, enums.VerticalAlignment] = None,
        ) -> None:
        Spanner.__init__(self)
        direction_ = String.to_tridirectional_lilypond_symbol(direction)
        self._direction = direction_

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, argument):
        return self._at_least_two_leaves(argument)

    def _copy_keywords(self, new):
        new._direction = self.direction

    def _get_lilypond_format_bundle(self, leaf):
        bundle = self._get_basic_lilypond_format_bundle(leaf)
        if self._is_my_only(leaf):
            return bundle
        assert 1 < len(self)
        if leaf is self[0]:
            strings = self._tweaked_start_command_strings()
            bundle.after.spanner_starts.extend(strings)
        elif leaf is self[-1]:
            string = self._stop_command_string()
            bundle.after.spanner_stops.append(string)
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self) -> typing.Optional[String]:
        r"""
        Gets direction.

        ..  container:: example

            Positions phrasing slur above staff:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> slur = abjad.PhrasingSlur(direction=abjad.Up)
            >>> abjad.attach(slur, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    ^ \(
                    d'8
                    e'8
                    f'8
                    \)
                }

        ..  container:: example

            Positions phrasing slur below staff:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> slur = abjad.PhrasingSlur(direction=abjad.Down)
            >>> abjad.attach(slur, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    _ \(
                    d'8
                    e'8
                    f'8
                    \)
                }

        ..  container:: example

            Positions phrasing slur according to LilyPond defaults:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> slur = abjad.PhrasingSlur(direction=None)
            >>> abjad.attach(slur, staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    \(
                    d'8
                    e'8
                    f'8
                    \)
                }

        """
        return self._direction



import copy
import math
import re
import typing
from abjad import exceptions
from abjad import mathtools
from abjad.system.AbjadObject import AbjadObject
try:
    from quicktions import Fraction  # type: ignore
except ImportError:
    from fractions import Fraction


class Duration(AbjadObject, Fraction):
    """
    Duration.

    ..  container:: example

        Initializes from integer numerator:

        >>> abjad.Duration(3)
        Duration(3, 1)

    ..  container:: example

        Initializes from integer numerator and denominator:

        >>> abjad.Duration(3, 16)
        Duration(3, 16)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator:

        >>> abjad.Duration(3.0)
        Duration(3, 1)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator and denominator:

        >>> abjad.Duration(3.0, 16)
        Duration(3, 16)

    ..  container:: example

        Initializes from integer-equivalent singleton:

        >>> abjad.Duration((3,))
        Duration(3, 1)

    ..  container:: example

        Initializes from integer-equivalent pair:

        >>> abjad.Duration((3, 16))
        Duration(3, 16)

    ..  container:: example

        Initializes from other duration:

        >>> abjad.Duration(abjad.Duration(3, 16))
        Duration(3, 16)

    ..  container:: example

        Intializes from fraction:

        >>> abjad.Duration(abjad.Fraction(3, 16))
        Duration(3, 16)

    ..  container:: example

        Initializes from solidus string:

        >>> abjad.Duration('3/16')
        Duration(3, 16)

    ..  container:: example

        Initializes from nonreduced fraction:

        >>> abjad.Duration(abjad.NonreducedFraction(6, 32))
        Duration(3, 16)

    ..  container:: example

        Durations inherit from built-in fraction:

        >>> isinstance(abjad.Duration(3, 16), abjad.Fraction)
        True

    ..  container:: example

        Durations are numbers:

        >>> import numbers

        >>> isinstance(abjad.Duration(3, 16), numbers.Number)
        True

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### CONSTRUCTOR ###

    def __new__(class_, *arguments):
        if len(arguments) == 1:
            argument = arguments[0]
            if type(argument) is class_:
                return argument
            if isinstance(argument, mathtools.NonreducedFraction):
                return Fraction.__new__(class_, *argument.pair)
            try:
                return Fraction.__new__(class_, *argument)
            except (AttributeError, TypeError):
                pass
            try:
                return Fraction.__new__(class_, argument)
            except (AttributeError, TypeError):
                pass
            if (isinstance(argument, tuple) and
                len(argument) == 2 and
                mathtools.is_integer_equivalent(argument[0]) and
                mathtools.is_integer_equivalent(argument[1]) and
                not argument[1] == 0):
                return Fraction.__new__(
                    class_,
                    int(argument[0]),
                    int(argument[1]),
                    )
            try:
                return Fraction.__new__(class_, argument.duration)
            except AttributeError:
                pass
            if isinstance(argument, str) and '/' not in argument:
                result = Duration._initialize_from_lilypond_duration_string(
                    argument)
                return Fraction.__new__(class_, result)
            if (isinstance(argument, tuple) and
                len(argument) == 1 and
                mathtools.is_integer_equivalent(argument[0])):
                return Fraction.__new__(class_, int(argument[0]))
        else:
            try:
                return Fraction.__new__(class_, *arguments)
            except TypeError:
                pass
            if mathtools.all_are_integer_equivalent_numbers(arguments):
                return Fraction.__new__(
                    class_,
                    *[int(x) for x in arguments]
                    )
        message = 'can not construct duration: {!r}.'
        message = message.format(arguments)
        raise ValueError(message)

    ### SPECIAL METHODS ###

    def __abs__(self, *arguments):
        """
        Gets absolute value of duration.

        Returns nonnegative duration.
        """
        return type(self)(Fraction.__abs__(self, *arguments))

    def __add__(self, *arguments):
        """
        Adds duration to ``arguments``.

        ..  container:: example

            Returns duration when ``arguments`` is a duration:

            >>> duration_1 = abjad.Duration(1, 2)
            >>> duration_2 = abjad.Duration(3, 2)
            >>> duration_1 + duration_2
            Duration(2, 1)

        ..  container:: example

            Returns nonreduced fraction when ``arguments`` is a nonreduced
            fraction:

            >>> duration = abjad.Duration(1, 2)
            >>> nonreduced_fraction = abjad.NonreducedFraction(3, 6)
            >>> duration + nonreduced_fraction
            NonreducedFraction(6, 6)

        Returns duration.
        """
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            result = arguments[0].__radd__(self)
        else:
            result = type(self)(Fraction.__add__(self, *arguments))
        return result

    def __div__(self, *arguments):
        """
        Divides duration by ``arguments``.

        ..  container:: example

            >>> abjad.Duration(1) / abjad.NonreducedFraction(3, 3)
            NonreducedFraction(3, 3)


        >>> abjad.NonreducedFraction(3, 3) / abjad.Duration(1)
        NonreducedFraction(3, 3)

        Returns multiplier.
        """
        import abjad
        if len(arguments) == 1 and isinstance(arguments[0], type(self)):
            fraction = Fraction.__truediv__(self, *arguments)
            result = abjad.Multiplier(fraction)
        elif len(arguments) == 1 and isinstance(
            arguments[0], abjad.NonreducedFraction):
            result = arguments[0].__rdiv__(self)
        else:
            result = type(self)(Fraction.__truediv__(self, *arguments))
        return result

    def __divmod__(self, *arguments):
        """
        Equals the pair (duration // ``arguments``, duration % ``arguments``).

        Returns pair.
        """
        truncated, residue = Fraction.__divmod__(self, *arguments)
        truncated = type(self)(truncated)
        residue = type(self)(residue)
        return truncated, residue

    def __eq__(self, argument):
        """
        Is true when duration equals ``argument``.

        Returns true or false.
        """
        return Fraction.__eq__(self, argument)

    def __format__(self, format_specification=''):
        """
        Formats duration.

        Set ``format_specification`` to ``''`` or ``'storage'``.
        Interprets ``''`` equal to ``'storage'``.

        Returns string.
        """
        import abjad
        if format_specification in ('', 'storage'):
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __ge__(self, argument):
        """
        Is true when duration is greater than or equal to ``argument``.

        Returns true or false.
        """
        return Fraction.__ge__(self, argument)

    def __gt__(self, argument):
        """
        Is true when duration is greater than ``argument``.

        Returns true or false.
        """
        return Fraction.__gt__(self, argument)

    def __hash__(self):
        """
        Hashes duration.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    def __le__(self, argument):
        """
        Is true when duration is less than or equal to ``argument``.

        Returns true or false.
        """
        return Fraction.__le__(self, argument)

    def __lt__(self, argument):
        """
        Is true when duration is less than ``argument``.

        Returns true or false.
        """
        return Fraction.__lt__(self, argument)

    def __mod__(self, *arguments):
        """
        Modulus operator applied to duration.

        Returns duration.
        """
        return type(self)(Fraction.__mod__(self, *arguments))

    def __mul__(self, *arguments):
        """
        Duration multiplied by ``arguments``.

        ..  container:: example

            Returns a new duration when ``arguments`` is a duration:

            >>> duration_1 = abjad.Duration(1, 2)
            >>> duration_2 = abjad.Duration(3, 2)
            >>> duration_1 * duration_2
            Duration(3, 4)

        ..  container:: example

            Returns nonreduced fraction when ``arguments`` is a nonreduced
            fraction:

            >>> duration = abjad.Duration(1, 2)
            >>> nonreduced_fraction = abjad.NonreducedFraction(3, 6)
            >>> duration * nonreduced_fraction
            NonreducedFraction(3, 12)

        Returns duration or nonreduced fraction.
        """
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            result = arguments[0].__rmul__(self)
        else:
            result = type(self)(Fraction.__mul__(self, *arguments))
        return result

    def __neg__(self, *arguments):
        """
        Negates duration.

        Returns new duration.
        """
        return type(self)(Fraction.__neg__(self, *arguments))

    def __pos__(self, *arguments):
        """
        Get positive duration.

        Returns new duration.
        """
        return type(self)(Fraction.__pos__(self, *arguments))

    def __pow__(self, *arguments):
        """
        Raises duration to ``arguments`` power.

        Returns new duration.
        """
        return type(self)(Fraction.__pow__(self, *arguments))

    def __radd__(self, *arguments):
        """
        Adds ``arguments`` to duration.

        Returns new duration.
        """
        return type(self)(Fraction.__radd__(self, *arguments))

    def __rdiv__(self, *arguments):
        """
        Divides ``arguments`` by duration.

        Returns new duration.
        """
        return type(self)(Fraction.__rdiv__(self, *arguments))

    def __rdivmod__(self, *arguments):
        """
        Documentation required.
        """
        return type(self)(Fraction.__rdivmod__(self, *arguments))

    def __reduce__(self):
        """
        Documentation required.
        """
        return type(self), (self.numerator, self.denominator)

    def __reduce_ex__(self, protocol):
        """
        Documentation required.
        """
        return type(self), (self.numerator, self.denominator)

    def __rmod__(self, *arguments):
        """
        Documentation required.
        """
        return type(self)(Fraction.__rmod__(self, *arguments))

    def __rmul__(self, *arguments):
        """
        Multiplies ``arguments`` by duration.

        Returns new duration.
        """
        return type(self)(Fraction.__rmul__(self, *arguments))

    def __rpow__(self, *arguments):
        """
        Raises ``arguments`` to the power of duration.

        Returns new duration.
        """
        return type(self)(Fraction.__rpow__(self, *arguments))

    def __rsub__(self, *arguments):
        """
        Subtracts duration from ``arguments``.

        Returns new duration.
        """
        return type(self)(Fraction.__rsub__(self, *arguments))

    def __rtruediv__(self, *arguments):
        """
        Documentation required.

        Returns new duration.
        """
        return type(self)(Fraction.__rtruediv__(self, *arguments))

    def __sub__(self, *arguments):
        """
        Subtracts ``arguments`` from duration.

        ..  container:: example

            >>> abjad.Duration(1, 2) - abjad.NonreducedFraction(2, 8)
            NonreducedFraction(2, 8)

            >>> abjad.NonreducedFraction(4, 8) - abjad.Duration(1, 4)
            NonreducedFraction(2, 8)

        Returns new duration.
        """
        if (
            len(arguments) == 1 and
            isinstance(arguments[0], mathtools.NonreducedFraction)
            ):
            return arguments[0].__rsub__(self)
        else:
            return type(self)(Fraction.__sub__(self, *arguments))

    def __truediv__(self, *arguments):
        """
        Documentation required.
        """
        return self.__div__(*arguments)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        return abjad.FormatSpecification(
            client=self,
            storage_format_args_values=[
                self.numerator,
                self.denominator,
                ],
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    @staticmethod
    def _group_by_implied_prolation(durations):
        durations = [
            mathtools.NonreducedFraction(duration)
            for duration in durations
            ]
        assert 0 < len(durations)
        group = [durations[0]]
        result = [group]
        for d in durations[1:]:
            d_f = set(mathtools.factors(d.denominator))
            d_f.discard(2)
            gd_f = set(mathtools.factors(group[0].denominator))
            gd_f.discard(2)
            if d_f == gd_f:
                group.append(d)
            else:
                group = [d]
                result.append(group)
        return result

    @staticmethod
    def _initialize_from_lilypond_duration_string(duration_string):
        numeric_body_strings = [str(2 ** n) for n in range(8)]
        other_body_strings = [r'\\breve', r'\\longa', r'\\maxima']
        body_strings = numeric_body_strings + other_body_strings
        body_strings = '|'.join(body_strings)
        pattern = r'^(%s)(\.*)$' % body_strings
        match = re.match(pattern, duration_string)
        if match is None:
            message = 'incorrect duration string format: {!r}.'
            message = message.format(duration_string)
            raise TypeError(message)
        body_string, dots_string = match.groups()
        try:
            body_denominator = int(body_string)
            body_duration = Fraction(1, body_denominator)
        except ValueError:
            if body_string == r'\breve':
                body_duration = Fraction(2)
            elif body_string == r'\longa':
                body_duration = Fraction(4)
            elif body_string == r'\maxima':
                body_duration = Fraction(8)
            else:
                message = 'unknown body string: {!r}.'
                message = message.format(body_string)
                raise ValueError(message)
        rational = body_duration
        for n in range(len(dots_string)):
            exponent = n + 1
            denominator = 2 ** exponent
            multiplier = Fraction(1, denominator)
            addend = multiplier * body_duration
            rational += addend
        return rational

    @staticmethod
    def _least_power_of_two_greater_equal(n, i=0):
        """
        When ``i = 2``, returns the second integer power of 2 greater than
        the least integer power of 2 greater than or equal to ``n``, and, in
        general, return the ``i`` th integer power of 2 greater than the least
        integer power of 2 greater than or equal to ``n``.

        Returns integer.
        """
        assert isinstance(n, (int, float, Fraction)), repr(n)
        assert 0 <= n, repr(n)
        result = 2 ** (int(math.ceil(math.log(n, 2))) + i)
        return result

    @staticmethod
    def _make_markup_score_block(selection):
        import abjad
        selection = copy.deepcopy(selection)
        staff = abjad.Staff(selection)
        staff.lilypond_type = 'RhythmicStaff'
        staff.remove_commands.append('Time_signature_engraver')
        staff.remove_commands.append('Staff_symbol_engraver')
        abjad.override(staff).stem.direction = abjad.Up
        abjad.override(staff).stem.length = 5
        abjad.override(staff).tuplet_bracket.bracket_visibility = True
        abjad.override(staff).tuplet_bracket.direction = abjad.Up
        abjad.override(staff).tuplet_bracket.minimum_length = 4
        abjad.override(staff).tuplet_bracket.padding = 1.25
        abjad.override(staff).tuplet_bracket.shorten_pair = (-1, -1.5)
        scheme = abjad.Scheme('ly:spanner::set-spacing-rods')
        abjad.override(staff).tuplet_bracket.springs_and_rods = scheme
        abjad.override(staff).tuplet_number.font_size = 0
        scheme = abjad.Scheme('tuplet-number::calc-fraction-text')
        abjad.override(staff).tuplet_number.text = scheme
        abjad.setting(staff).tuplet_full_length = True
        layout_block = abjad.Block(name='layout')
        layout_block.indent = 0
        layout_block.ragged_right = True
        score = abjad.Score([staff])
        abjad.override(score).spacing_spanner.spacing_increment = 0.5
        abjad.setting(score).proportional_notation_duration = False
        return score, layout_block

    @staticmethod
    def _to_score_markup(selection):
        import abjad
        staff, layout_block = Duration._make_markup_score_block(selection)
        command = abjad.MarkupCommand('score', [staff, layout_block])
        markup = abjad.Markup(command)
        return markup

    ### PUBLIC PROPERTIES ###

    @property
    def dot_count(self):
        r"""
        Gets dot count.

        ..  container:: example

            Gets dot count:

            >>> for n in range(1, 16 + 1):
            ...     try:
            ...         duration = abjad.Duration(n, 16)
            ...         sixteenths = duration.with_denominator(16)
            ...         dot_count = duration.dot_count
            ...         string = f'{sixteenths!s}\t{dot_count}'
            ...         print(string)
            ...     except abjad.AssignabilityError:
            ...         sixteenths = duration.with_denominator(16)
            ...         print(f'{sixteenths!s}\t--')
            ...
            1/16    0
            2/16    0
            3/16    1
            4/16    0
            5/16    --
            6/16    1
            7/16    2
            8/16    0
            9/16    --
            10/16   --
            11/16   --
            12/16   1
            13/16   --
            14/16   2
            15/16   3
            16/16   0

        Dot count defined equal to number of dots required to notate duration.

        Raises assignability error when duration is not assignable.

        Returns positive integer.
        """
        if not self.is_assignable:
            raise exceptions.AssignabilityError
        binary_string = mathtools.integer_to_binary_string(self.numerator)
        digit_sum = sum([int(x) for x in list(binary_string)])
        dot_count = digit_sum - 1
        return dot_count

    @property
    def equal_or_greater_assignable(self):
        r"""
        Gets assignable duration equal to or just greater than this
        duration.

        ..  container:: example

            Gets equal-or-greater assignable duration:

            >>> for numerator in range(1, 16 + 1):
            ...     duration = abjad.Duration(numerator, 16)
            ...     result = duration.equal_or_greater_assignable
            ...     sixteenths = duration.with_denominator(16)
            ...     print(f'{sixteenths!s}\t{result!s}')
            ...
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    3/8
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    3/4
            10/16   3/4
            11/16   3/4
            12/16   3/4
            13/16   7/8
            14/16   7/8
            15/16   15/16
            16/16   1

        Returns new duration.
        """
        good_denominator = mathtools.greatest_power_of_two_less_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator += 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_greater_power_of_two(self):
        r"""
        Gets duration equal or just greater power of two.

        ..  container:: example

            Gets equal-or-greater power-of-two:

            >>> for numerator in range(1, 16 + 1):
            ...     duration = abjad.Duration(numerator, 16)
            ...     result = duration.equal_or_greater_power_of_two
            ...     sixteenths = duration.with_denominator(16)
            ...     print(f'{sixteenths!s}\t{result!s}')
            ...
            1/16    1/16
            2/16    1/8
            3/16    1/4
            4/16    1/4
            5/16    1/2
            6/16    1/2
            7/16    1/2
            8/16    1/2
            9/16    1
            10/16   1
            11/16   1
            12/16   1
            13/16   1
            14/16   1
            15/16   1
            16/16   1

        Returns new duration.
        """
        denominator_exponent = -int(math.ceil(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def equal_or_lesser_assignable(self):
        r"""
        Gets assignable duration equal or just less than this duration.

        ..  container:: example

            Gets equal-or-lesser assignable duration:

            >>> for numerator in range(1, 16 + 1):
            ...     duration = abjad.Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_assignable
            ...     sixteenths = duration.with_denominator(16)
            ...     print(f'{sixteenths!s}\t{result!s}')
            ...
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    1/4
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   3/4
            13/16   3/4
            14/16   7/8
            15/16   15/16
            16/16   1

        Returns new duration.
        """
        good_denominator = self._least_power_of_two_greater_equal(
            self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator -= 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_lesser_power_of_two(self):
        r"""
        Gets duration of the form ``d**2`` equal to or just less than this
        duration.

        ..  container:: example

            Gets equal-or-lesser power-of-two:

            >>> for numerator in range(1, 16 + 1):
            ...     duration = abjad.Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_power_of_two
            ...     sixteenths = duration.with_denominator(16)
            ...     print(f'{sixteenths!s}\t{result!s}')
            ...
            1/16    1/16
            2/16    1/8
            3/16    1/8
            4/16    1/4
            5/16    1/4
            6/16    1/4
            7/16    1/4
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   1/2
            13/16   1/2
            14/16   1/2
            15/16   1/2
            16/16   1

        Returns new duration.
        """
        denominator_exponent = -int(math.floor(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def flag_count(self):
        r"""
        Gets flag count.

        ..  container:: example

            Gets flag count:

            >>> for n in range(1, 16 + 1):
            ...     duration = abjad.Duration(n, 64)
            ...     sixty_fourths = duration.with_denominator(64)
            ...     print(f'{sixty_fourths!s}\t{duration.flag_count}')
            ...
            1/64    4
            2/64    3
            3/64    3
            4/64    2
            5/64    2
            6/64    2
            7/64    2
            8/64    1
            9/64    1
            10/64   1
            11/64   1
            12/64   1
            13/64   1
            14/64   1
            15/64   1
            16/64   0

        Flag count defined equal to number of flags required to notate
        duration.

        Returns nonnegative integer.
        """
        log = math.log(float(self.numerator) / self.denominator, 2)
        count = -int(math.floor(log)) - 2
        return max(count, 0)

    @property
    def has_power_of_two_denominator(self):
        r"""
        Is true when duration is an integer power of two.

        ..  container:: example

            Is true when duration has power-of-two denominator:

            >>> for n in range(1, 16 + 1):
            ...     duration = abjad.Duration(1, n)
            ...     result = duration.has_power_of_two_denominator
            ...     print('{!s}\t{}'.format(duration, result))
            ...
            1       True
            1/2     True
            1/3     False
            1/4     True
            1/5     False
            1/6     False
            1/7     False
            1/8     True
            1/9     False
            1/10    False
            1/11    False
            1/12    False
            1/13    False
            1/14    False
            1/15    False
            1/16    True

        Returns true or false.
        """
        exponent = math.log(self.denominator, 2)
        return int(exponent) == exponent

    @property
    def implied_prolation(self):
        r"""
        Gets implied prolation.

        ..  container:: example

            Gets implied prolation:

            >>> for denominator in range(1, 16 + 1):
            ...     duration = abjad.Duration(1, denominator)
            ...     result = duration.implied_prolation
            ...     print('{!s}\t{!s}'.format(duration, result))
            ...
            1       1
            1/2     1
            1/3     2/3
            1/4     1
            1/5     4/5
            1/6     2/3
            1/7     4/7
            1/8     1
            1/9     8/9
            1/10    4/5
            1/11    8/11
            1/12    2/3
            1/13    8/13
            1/14    4/7
            1/15    8/15
            1/16    1

        Returns multipler.
        """
        import abjad
        numerator = \
            mathtools.greatest_power_of_two_less_equal(self.denominator)
        return abjad.Multiplier(numerator, self.denominator)

    @property
    def is_assignable(self):
        r"""
        Is true when duration is assignable.

        ..  container:: example

            Is true when duration is assignable:

            >>> for numerator in range(0, 16 + 1):
            ...     duration = abjad.Duration(numerator, 16)
            ...     sixteenths = duration.with_denominator(16)
            ...     print('{!s}\t{}'.format(sixteenths, duration.is_assignable))
            ...
            0/16    False
            1/16    True
            2/16    True
            3/16    True
            4/16    True
            5/16    False
            6/16    True
            7/16    True
            8/16    True
            9/16    False
            10/16   False
            11/16   False
            12/16   True
            13/16   False
            14/16   True
            15/16   True
            16/16   True

        Returns true or false.
        """
        if 0 < self < 16:
            if mathtools.is_nonnegative_integer_power_of_two(
                self.denominator):
                if mathtools.is_assignable_integer(self.numerator):
                    return True
        return False

    @property
    def lilypond_duration_string(self):
        """
        Gets LilyPond duration string.

        ..  container:: example

            Gets LilyPond duration string:

                >>> abjad.Duration(3, 16).lilypond_duration_string
                '8.'

        Raises assignability error when duration is not assignable.

        Returns string.
        """
        if not self.is_assignable:
            raise exceptions.AssignabilityError(self)
        undotted_rational = self.equal_or_lesser_power_of_two
        if undotted_rational <= 1:
            undotted_duration_string = str(undotted_rational.denominator)
        elif undotted_rational == type(self)(2, 1):
            undotted_duration_string = r'\breve'
        elif undotted_rational == type(self)(4, 1):
            undotted_duration_string = r'\longa'
        elif undotted_rational == type(self)(8, 1):
            undotted_duration_string = r'\maxima'
        else:
            message = 'can not process undotted rational: {}'
            message = message.format(undotted_rational)
            raise ValueError(message)
        dot_count = self.dot_count
        dot_string = '.' * dot_count
        dotted_duration_string = undotted_duration_string + dot_string
        return dotted_duration_string

    @property
    def pair(self):
        """
        Gets numerator and denominator.

        ..  container:: example

            Gets pair:

            >>> abjad.Duration(3, 16).pair
            (3, 16)

        Returns integer pair.
        """
        return self.numerator, self.denominator

    @property
    def prolation_string(self):
        """
        Gets prolation string.

        ..  container:: example

            Gets prolation string:

            >>> abjad.Duration(3, 16).prolation_string
            '16:3'

        Returns string.
        """
        return f'{self.denominator}:{self.numerator}'

    @property
    def reciprocal(self):
        """
        Gets reciprocal.

        ..  container:: example

            Gets reciprocal:

            >>> abjad.Duration(3, 7).reciprocal
            Duration(7, 3)

        Returns new duration.
        """
        return type(self)(self.denominator, self.numerator)

    ### PUBLIC FUNCTIONS ###

    @staticmethod
    def durations_to_nonreduced_fractions(
        durations: typing.List,
        ) -> typing.List[mathtools.NonreducedFraction]:
        """
        Changes ``durations`` to nonreduced fractions sharing least common
        denominator.

        ..  container:: example

            Changes durations to nonreduced fractions:

            >>> durations = [abjad.Duration(2, 4), 3, (5, 16)]
            >>> result = abjad.Duration.durations_to_nonreduced_fractions(durations)
            >>> for x in result:
            ...     x
            ...
            NonreducedFraction(8, 16)
            NonreducedFraction(48, 16)
            NonreducedFraction(5, 16)

        """
        durations_ = [Duration(_) for _ in durations]
        denominators = [_.denominator for _ in durations_]
        lcd = mathtools.least_common_multiple(*denominators)
        nonreduced_fractions = [
            mathtools.NonreducedFraction(_).with_denominator(lcd)
            for _ in durations_
            ]
        return nonreduced_fractions

    @staticmethod
    def from_lilypond_duration_string(lilypond_duration_string):
        r"""
        Initializes duration from LilyPond duration string.

        ..  container:: example

            Initializes duration from LilyPond duration string:

            >>> abjad.Duration.from_lilypond_duration_string('8.')
            Duration(3, 16)

        Returns duration.
        """
        fraction = Duration._initialize_from_lilypond_duration_string(
            lilypond_duration_string)
        return Duration(fraction)

    @staticmethod
    def is_token(argument):
        """
        Is true when ``argument`` correctly initializes a duration.

        ..  container:: example

            Is true when expression is a duration token:

            >>> abjad.Duration.is_token('8.')
            True

        Returns true or false.
        """
        try:
            Duration.__new__(Duration, argument)
            return True
        except:
            return False

    def to_clock_string(self):
        r"""
        Changes duration to clock string.

        ..  container:: example

            Changes duration to clock string:

            >>> note = abjad.Note("c'4")
            >>> duration = abjad.Duration(117)
            >>> clock_string = duration.to_clock_string()
            >>> clock_string
            "1'57''"

            >>> string = '"{}"'.format(clock_string)
            >>> markup = abjad.Markup(string, direction=abjad.Up)
            >>> abjad.attach(markup, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                ^ \markup { 1'57'' }

        Rounds down to nearest second.

        Returns string.
        """
        minutes = int(self / 60)
        seconds = str(int(self - minutes * 60)).zfill(2)
        clock_string = f"{minutes}'{seconds}''"
        return clock_string

    def to_score_markup(self):
        r"""
        Changes duration to score markup.

        ..  container:: example

            Changes assignable duration to score markup:

            >>> markup = abjad.Duration(3, 16).to_score_markup()
            >>> abjad.show(markup) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(markup)
                \markup {
                    \score
                        {
                            \new Score
                            \with
                            {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            }
                            <<
                                \new RhythmicStaff
                                \with
                                {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.minimum-length = #4
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                                    \override TupletNumber.font-size = #0
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                }
                                {
                                    c'8.
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                    }

        ..  container:: example

            Changes nonassignable duration to score markup:

            >>> markup = abjad.Duration(5, 16).to_score_markup()
            >>> abjad.show(markup) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(markup)
                \markup {
                    \score
                        {
                            \new Score
                            \with
                            {
                                \override SpacingSpanner.spacing-increment = #0.5
                                proportionalNotationDuration = ##f
                            }
                            <<
                                \new RhythmicStaff
                                \with
                                {
                                    \remove Time_signature_engraver
                                    \remove Staff_symbol_engraver
                                    \override Stem.direction = #up
                                    \override Stem.length = #5
                                    \override TupletBracket.bracket-visibility = ##t
                                    \override TupletBracket.direction = #up
                                    \override TupletBracket.minimum-length = #4
                                    \override TupletBracket.padding = #1.25
                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                    \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                                    \override TupletNumber.font-size = #0
                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                    tupletFullLength = ##t
                                }
                                {
                                    c'4
                                    ~
                                    c'16
                                }
                            >>
                            \layout {
                                indent = #0
                                ragged-right = ##t
                            }
                        }
                    }

        ..  container:: example

            Override tuplet number text like this:

            >>> tuplet = abjad.Tuplet((5, 7), "c'16 c' c' c' c' c' c'")
            >>> abjad.attach(abjad.Beam(), tuplet[:])
            >>> staff = abjad.Staff([tuplet], lilypond_type='RhythmicStaff')
            >>> duration = abjad.inspect(tuplet).duration()
            >>> markup = duration.to_score_markup()
            >>> markup = markup.scale((0.75, 0.75))
            >>> abjad.override(tuplet).tuplet_number.text = markup
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new RhythmicStaff
                {
                    \override TupletNumber.text = \markup {
                        \scale
                            #'(0.75 . 0.75)
                            \score
                                {
                                    \new Score
                                    \with
                                    {
                                        \override SpacingSpanner.spacing-increment = #0.5
                                        proportionalNotationDuration = ##f
                                    }
                                    <<
                                        \new RhythmicStaff
                                        \with
                                        {
                                            \remove Time_signature_engraver
                                            \remove Staff_symbol_engraver
                                            \override Stem.direction = #up
                                            \override Stem.length = #5
                                            \override TupletBracket.bracket-visibility = ##t
                                            \override TupletBracket.direction = #up
                                            \override TupletBracket.minimum-length = #4
                                            \override TupletBracket.padding = #1.25
                                            \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                            \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                                            \override TupletNumber.font-size = #0
                                            \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                            tupletFullLength = ##t
                                        }
                                        {
                                            c'4
                                            ~
                                            c'16
                                        }
                                    >>
                                    \layout {
                                        indent = #0
                                        ragged-right = ##t
                                    }
                                }
                        }
                    \times 5/7 {
                        c'16
                        [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        ]
                    }
                    \revert TupletNumber.text
                }

        Returns markup.
        """
        import abjad
        maker = abjad.LeafMaker()
        notes = maker([0], [self])
        markup = self._to_score_markup(notes)
        return markup

    def with_denominator(self, denominator):
        """
        Changes duration to nonreduced fraction with ``denominator``.

        ..  container:: example

            Changes duration to nonreduced fraction:

            >>> duration = abjad.Duration(1, 4)
            >>> for denominator in (4, 8, 16, 32):
            ...     print(duration.with_denominator(denominator))
            ...
            1/4
            2/8
            4/16
            8/32

        Returns new duration.
        """
        nonreduced_fraction = mathtools.NonreducedFraction(self)
        return nonreduced_fraction.with_denominator(denominator)
